/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include "iface_def.hpp"
#include "selektor.hpp"
#include "vypis.hpp"

FSOUND_SAMPLE *iface_def_snd[16];

int c_iface_def::init() {
    aBitmapGL galaxie;

    iface_def_snd[0] = FSOUND_Sample_Load( FSOUND_FREE, "data/iface.ogg",
                                           FSOUND_NORMAL, 0, 0 );
    iface_def_snd[1] = FSOUND_Sample_Load( FSOUND_FREE, "data/snd1.ogg",
                                           FSOUND_NORMAL, 0, 0 );

    int r = 128;
    void *mem = malloc(r*r*4);

    glGenTextures(1, &tab_texture);
    glBindTexture(GL_TEXTURE_2D, tab_texture);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8,r,r,0,GL_RGBA8,GL_UNSIGNED_BYTE,mem);
    
    for (int i=0; i<4; ++i) {
        glGenTextures(1, &pad[i].texture);
        glBindTexture(GL_TEXTURE_2D, pad[i].texture);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8,r,r,0,GL_RGBA8,GL_UNSIGNED_BYTE,mem);
    }
    
    return 0;
};

int c_iface_def::entry() {
    lParam = 0;
    
    for (int i=0; i<4; ++i) left_bat_charge[i] = 0.0;

    Nahoru("Milky Way Galaxy");
    sprintf(szDole,"");
    
    FSOUND_PlaySound( FSOUND_FREE, iface_def_snd[0] );
    
    selektor[0].length = 9.0;
    strcpy(selektor[0].name, "Hoessce System");
    selektor[0].apAlpha.addp(0.0, 0.0);
    selektor[0].apAlpha.addp(1.0, 1.0);
    selektor[0].apAlpha.addp(8.0, 1.0);
    selektor[0].apAlpha.addp(9.0, 0.0);
    selektor[0].apPos.addp(0.0,-0.50,-0.50);
    selektor[0].apPos.addp(1.0,-0.48,-0.48);
    selektor[0].apPos.addp(2.0,-0.05, 0.00);
    selektor[0].apPos.addp(8.0, 0.00,-0.01);
    selektor[0].apPos.addp(9.0, 0.20,-0.20);

    selektor[1].length = 9.0;
    strcpy(selektor[1].name, "Beta Lyrae");
    selektor[1].apAlpha.addp(0.0, 0.0);
    selektor[1].apAlpha.addp(1.0, 1.0);
    selektor[1].apAlpha.addp(8.0, 1.0);
    selektor[1].apAlpha.addp(9.0, 0.0);
    selektor[1].apPos.addp(0.0, 0.80,-0.00);
    selektor[1].apPos.addp(1.0, 0.70,-0.05);
    selektor[1].apPos.addp(2.0,-0.10,-0.60);
    selektor[1].apPos.addp(8.0,-0.03,-0.59);
    selektor[1].apPos.addp(9.0, 0.20,-0.70);

    selektor[2].length = 9.0;
    strcpy(selektor[2].name, "Sol System");
    selektor[2].apAlpha.addp(0.0, 0.0);
    selektor[2].apAlpha.addp(1.0, 1.0);
    selektor[2].apAlpha.addp(8.0, 1.0);
    selektor[2].apAlpha.addp(9.0, 0.0);
    selektor[2].apPos.addp(0.0, 0.80, 0.00);
    selektor[2].apPos.addp(1.0, 0.70, 0.05);
    selektor[2].apPos.addp(2.0,-0.15, 0.60);
    selektor[2].apPos.addp(8.0,-0.22, 0.58);
    selektor[2].apPos.addp(9.0,-0.40, 0.30);
    
    b_slunce_vybrane = false;
   
    tab_alpha = 0.0;

    progress_alpha = 0.0;
    progress = 0.0;
    black = 0.0;

    for (int i=0; i<4; ++i) pad[i].alpha = 0.0;

    FSOUND_SetVolume(0,64);
    
    return 0;
};

int c_iface_def::close() {
    return 0;
};

int c_iface_def::evaluate(int time) {
    lParam += time;

    if (lParam < 20000) {
        if (left_bat_charge[0] < 1.0) left_bat_charge[0] += 0.0004*time;
        if (left_bat_charge[1] < 1.0) left_bat_charge[1] += 0.0002*time;
        if (left_bat_charge[2] < 1.0) left_bat_charge[2] += 0.0001*time;
        if (left_bat_charge[3] < 1.0) left_bat_charge[3] += 0.00005*time;
    } else {
        float overal = (lParam-20000.0)/300.0;
    
        left_bat_charge[0] = (overal/250.0) + (overal/10000.0)
                           * sin( 0.01*lParam + PI/2.0 );
        left_bat_charge[1] = (overal/500.0) + (overal/10000.0)
                           * sin( 0.01*lParam + PI );
        left_bat_charge[2] = (overal/750.0) + (overal/10000.0) 
                           * sin( 0.01*lParam + 3.0*PI/2.0 );
        left_bat_charge[3] = (overal/1000.0) + (overal/10000.0)     
                           * sin( 0.01*lParam );
        
        for (int i=0; i<4; ++i) 
            if (left_bat_charge[i] > 1.0) left_bat_charge[i] = 1.0;
    }
    
    if (lParam > 1000 && lParam < 3000 && !selektor[0].enabled) {
        selektor[0].enabled = true;
    }
    if (lParam > 5000 && lParam < 7000 && !selektor[1].enabled) {
        selektor[1].enabled = true;
    }
    if (lParam > 9000 && lParam < 11000 && !selektor[2].enabled) {
        selektor[2].enabled = true;
    }
    if (lParam > 15000 && !b_slunce_vybrane) {
        b_slunce_vybrane = true;
        Dolu("Sol System Confirmed");
    }

    
    for (int i=0; i<8; ++i)
        left_quad_bar[i] = 0.5f+0.5f*cos( 0.001*(lParam+320*(14-i)) );
    
    right_top = 1.0;

    for (int i=0; i<8; ++i)
        if (selektor[i].enabled) {
            selektor[i].time += 0.001*time;
            if (selektor[i].time > selektor[i].sndpt && !selektor[i].sndptpassed) {
                selektor[i].sndptpassed = true;
                FSOUND_PlaySound( FSOUND_FREE, iface_def_snd[1] );
            }
            if (selektor[i].time > selektor[i].length) {
                selektor[i].enabled = false;
                selektor[i].reset();
            }
        }
    
    return 0;
};

int c_iface_def::render() {
    glPushAttrib( GL_COLOR_BUFFER_BIT|GL_CURRENT_BIT|GL_ENABLE_BIT );
 
    glLoadIdentity();
    glDisable(GL_LIGHTING);

    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    glEnable(GL_COLOR_MATERIAL);
    glEnable(GL_LINE_SMOOTH);
    glLineWidth(2.8f);
    
    // pruhy s 0.5 cernoty :-)
    glColor4f(0.0,0.0,0.0, 0.5*(1.0-black));
    glRecti( 0, 0, maxx, 50 ); // nahore a dole
    glRecti( 0, maxy-50, maxx, maxy );
    glRecti( 42, 82, 298, 118 );
    
    glBegin(GL_POLYGON);
        glColor4f(0.0,0.0,0.0, 0.5*tab_alpha*(1.0-black));
        glVertex2i(40,164);
        glVertex2i(166,164);
        glVertex2i(296,294);
        glVertex2i(296,((maxy-120)<?580)+10);
        glVertex2i(50,((maxy-120)<?580)+10);
        glVertex2i(40,(maxy-120)<?580);
        glVertex2i(40,164);
    glEnd();

    glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));
    glBegin(GL_LINES);
        glVertex2i(0,50); glVertex2i(maxx,50); // oddelovace nahore
        glVertex2i(0,maxy-50); glVertex2i(maxx,maxy-50); //  a dole
        
        for (int i=1; i<100; ++i) { // carky po okrajich
            int j = (i%5)?8:16;
            float p = ((float)i/100.0f)*(maxy-100)+50.0f;
            
            glVertex2f(0,p);    glVertex2f(j,p);
            glVertex2f(maxx,p); glVertex2f(maxx-j,p);
        }

        // vlevo nahore
        glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));glVertex2i(36,76);
            glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(296,76);
        glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));glVertex2i(36,76);
            glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(36,116);

        glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));
        glVertex2i(208,maxy-10); glVertex2i(208,maxy-42);
        glVertex2i(208,maxy-42); glVertex2i(maxx-208,maxy-42);
        glVertex2i(maxx-208,maxy-42); glVertex2i(maxx-208,maxy-10);

        // podtrzeni hlasky
        glVertex2i(20,maxy-50); glVertex2i(30,maxy-60);
        glVertex2i(30,maxy-60); glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(maxx/2,maxy-60);
        glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));

        // pro fps
        glVertex2i(maxx-400,50); glVertex2i(maxx-380,70);
        glVertex2i(maxx-380,70); glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(maxx-180,70);
        glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));

        
        for (int i=0; i<6; ++i) {
            glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));
            glVertex2i(maxx, 16+4*i);

            int val = (int)(200.0f*(right_top + 0.1f*sin( 0.01f*(lParam + 360.0f*i) )));
            glColor4f(1.0,0.8,0.0, 0.4*(1.0-black));
            glVertex2i(maxx - val, 16+4*i);
        }        

        // TAB obrys vn2j39
        glColor4f(1.0,0.8,0.0, 0.9*tab_alpha*(1.0-black));glVertex2i(36,160);
            glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(166,160);
        glColor4f(1.0,0.8,0.0, 0.9*tab_alpha*(1.0-black));glVertex2i(36,160);
            glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(36,(maxy-120)<?580);

        // progress obrys
        glColor4f(1.0,0.8,0.0, 0.9*progress_alpha*(1.0-black));glVertex2i(maxx-36,maxy-76);
            glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(maxx-166,maxy-76);
        glColor4f(1.0,0.8,0.0, 0.9*progress_alpha*(1.0-black));glVertex2i(maxx-36,maxy-76);
            glColor4f(1.0,0.8,0.0, 0.0); glVertex2i(maxx-36,76);
            
    glEnd();

    // pady vnejsi obrys
    int vyska1 = (maxy-160)/4;
    int vyska2 = 4*vyska1/5;
    for (int i=0; i<4; ++i) {
        if (!pad[i].alpha) continue; 

        glBegin(GL_LINES);
            glColor4f(1.0,0.8,0.0,0.0);
            glVertex2i(maxx - 4*vyska2/3, 80 + i*vyska1 - vyska1/20);

            glColor4f(1.0,0.8,0.0, 0.9*pad[i].alpha*(1.0-black));
            glVertex2i(maxx - 24,         80 + i*vyska1 - vyska1/20 );
            glVertex2i(maxx - 24,         80 + i*vyska1 - vyska1/20 );

            glColor4f(1.0,0.8,0.0,0.0);
            glVertex2i(maxx - 24,         80 + i*vyska1 + vyska2 );
        glEnd();
    };
//*/

    glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));


    // vlevo dole
    for (int i=0; i<4; ++i) {
        glBegin(GL_QUADS);
            int pix = 32 - (int)(32.0*left_bat_charge[i]);
            glVertex2i(48*i+8,maxy-42+pix); glVertex2i(48*i+40,maxy-42+pix);
            glVertex2i(48*i+40,maxy-10); glVertex2i(48*i+8,maxy-10);
        glEnd();
        glBegin(GL_LINE_LOOP);
            glVertex2i(48*i+8,maxy-42); glVertex2i(48*i+40,maxy-42);
            glVertex2i(48*i+40,maxy-10); glVertex2i(48*i+8,maxy-10);
        glEnd();
    }
    // vlevo nahore
    for (int i=0; i<8; ++i) {
        glColor4f(1.0,0.8,0.0, 0.9*left_quad_bar[i]*(1.0-black));
        glBegin(GL_QUADS);
            glVertex2i(48*i+8, 16); glVertex2i(48*i+48,16);
            glVertex2i(48*i+48,32); glVertex2i(48*i+8, 32);
        glEnd();
        glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));
        glBegin(GL_LINE_LOOP);
            glVertex2i(48*i+8, 16); glVertex2i(48*i+48,16);
            glVertex2i(48*i+48,32); glVertex2i(48*i+8, 32);
        glEnd();
    }

    for (int i=0; i<8; ++i)
        if (selektor[i].enabled) {
            double x, y, al;
            selektor[i].apPos.get( selektor[i].time, &x, &y );
            selektor[i].apAlpha.get( selektor[i].time, &al );
         
            glColor4f(1.0,0.8,0.0, 0.9 * al*(1.0-black));
            DrawSelektor((int)((maxx*(x+1))/2), maxy-(int)((maxy*(y+1))/2),
                         selektor[i].name);
        }



    glLineWidth(1.0f);
    glDisable(GL_LINE_SMOOTH);
    glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));
    glBegin(GL_LINES); 
        glVertex2i(0,46); glVertex2i(maxx,46); // oddelovace nahore
//        glVertex2i(0,maxy-45); glVertex2i(maxx,maxy-45); //  a dole

        // vlevo nahore
        glVertex2i(40,80); glVertex2i(290,80);
        glVertex2i(40,80); glVertex2i(40,110);
        glVertex2i(300,120); glVertex2i(300,90);
        glVertex2i(300,120); glVertex2i(50,120);
        glVertex2i(290,80); glVertex2i(300,90);
        glVertex2i(40,109); glVertex2i(51,120);

        // TAB obrys vnit5n9
        glColor4f(1.0,0.8,0.0, 0.9*tab_alpha*(1.0-black));
        glVertex2i(40,164); glVertex2i(166,164);
        glVertex2i(40,164); glVertex2i(40,((maxy-120)<?580)+1);
        glVertex2i(40,(maxy-120)<?580); glVertex2i(50,((maxy-120)<?580)+10);
        glVertex2i(50,((maxy-120)<?580)+10); glVertex2i(296,((maxy-120)<?580)+10);
        glVertex2i(166,164); glVertex2i(296,294);
        glVertex2i(296,295); glVertex2i(296,((maxy-120)<?580)+10);

    glEnd();

    // progress ctverecky
    int vyska = (maxy-160)/10;
    int pcbeg = (int)(10.0-progress*10.0f);
    for (int i=pcbeg; i<10; ++i) {
        float pomer = 0.0;
        if (i == pcbeg) pomer = 1.0 - (10.0f*fmodf(progress, 0.1f));
        
        glColor4f(1.0,0.8,0.0, 0.6*progress_alpha*(1.0-black));
        glRecti(maxx-160,100+vyska*(i+0)+(int)((vyska-12)*pomer),
                maxx-40 , 89+vyska*(i+1));

        glColor4f(1.0,0.8,0.0, 0.9*progress_alpha*(1.0-black));
        glBegin(GL_LINE_LOOP);
            glVertex2i(maxx-160,100+vyska*(i+0));
            glVertex2i(maxx-40 ,100+vyska*(i+0));
            glVertex2i(maxx-40 , 89+vyska*(i+1));
            glVertex2i(maxx-160, 89+vyska*(i+1));
        glEnd();
    }
                

    glEnable(GL_LINE_SMOOTH);
    glBegin(GL_LINES); 
        float polomer[4] = { 0.01, 0.005, 0.0035, 0.002 };
        glColor4f(1.0,0.8,0.0, 0.5*(1.0-black));
        for (int i=216; i<maxx-216; i+=16) {
            for (int j=0; j<4; ++j) {
                int y1 = (maxy-24)+(int)(14.0*sin( polomer[j]*(lParam+10*i) ));
                glVertex2i(i,y1); glVertex2i(i+16,y1);
            }
        };
    glEnd();
    glDisable(GL_LINE_SMOOTH);

    glColor4f(1.0,0.8,0.0, 0.9*(1.0-black));

    char aaa[256];
    if (right_x) sprintf(aaa,"X: %015.3f Km",10000.0f*right_x);
            else strcpy(aaa,"X: Data Unavilable");
    aGLPrintf(hAGL, sysfont, maxx-192 , maxy-34, aaa);
    if (right_y) sprintf(aaa,"Y: %015.3f Km",10000.0f*right_y);
            else strcpy(aaa,"Y: Data Unavilable");
    aGLPrintf(hAGL, sysfont, maxx-192 , maxy-22, aaa);
    
    if (right_z) sprintf(aaa,"Z: %015.3f Km",10000.0f*right_z);
            else strcpy(aaa,"Z: Synchronized Plane");
    aGLPrintf(hAGL, sysfont, maxx-192 , maxy-10, aaa);

    glColor4f(1.0,1.0,1.0, 1.0*(1.0-black));
    aGLPrintf(hAGL, midfont, 55, 107, szNahore);

    glColor4f(0.0,0.0,0.0, 1.0*(1.0-black));
    aGLPrintf(hAGL, bigfont, 40+2, maxy-70+2, szDole);
    glColor4f(1.0,1.0,1.0, 1.0*(1.0-black));
    aGLPrintf(hAGL, bigfont, 40, maxy-70, szDole);

    for (int i=0; i<4; ++i) {
        if (!pad[i].alpha) continue; 

        glColor4f(1.0,0.8,0.0, 1.0*(1.0-black)*pad[i].alpha);
        aGLPrintf(hAGL, font, maxx - 4*vyska2/3, 80 + i*vyska1 + vyska1/15, pad[i].sz);
    }


    // TAB texture
    glColor4f(1.0,1.0,1.0, 0.9*tab_alpha*(1.0-black));
    glEnable(GL_TEXTURE_2D);
    glBindTexture( GL_TEXTURE_2D, tab_texture );
    glBegin(GL_QUADS);
        glTexCoord2i(0,1); glVertex2i( 60, 200);
        glTexCoord2i(1,1); glVertex2i(160, 200);
        glTexCoord2i(1,0); glVertex2i(160, 275);
        glTexCoord2i(0,0); glVertex2i( 60, 275);
    glEnd();
    glDisable(GL_TEXTURE_2D);

    // PADS - obrys a textura
    for (int i=0; i<4; ++i) {
        if (!pad[i].alpha) continue; 
        
        glColor4f(1.0,1.0,1.0, 0.9*pad[i].alpha*(1.0-black));
        
        glEnable(GL_TEXTURE_2D);
        glBindTexture( GL_TEXTURE_2D, pad[i].texture );
        glBegin(GL_QUADS);
            glTexCoord2i(1,1); glVertex2i(maxx - 32,         80 + i*vyska1 + vyska1/10 );
            glTexCoord2i(0,1); glVertex2i(maxx - 4*vyska2/3, 80 + i*vyska1 + vyska1/10 );
            glTexCoord2i(0,0); glVertex2i(maxx - 4*vyska2/3, 80 + i*vyska1 + vyska2 );
            glTexCoord2i(1,0); glVertex2i(maxx - 32,         80 + i*vyska1 + vyska2 );
        glEnd();
        glDisable(GL_TEXTURE_2D);
    };
    for (int i=0; i<4; ++i) {
        if (!pad[i].alpha) continue; 

        glColor4f(1.0,0.8,0.0, 0.9*pad[i].alpha*(1.0-black));
        glBegin(GL_LINE_LOOP);
            glVertex2i(maxx - 32,         80 + i*vyska1 + vyska1/10 );
            glVertex2i(maxx - 4*vyska2/3, 80 + i*vyska1 + vyska1/10 );
            glVertex2i(maxx - 4*vyska2/3, 80 + i*vyska1 + vyska2 );
            glVertex2i(maxx - 32,         80 + i*vyska1 + vyska2 );
        glEnd();
    };//*/

    // TAB texts
    glScissor(55,0,220,maxy);
    glEnable(GL_SCISSOR_TEST);
    glColor4f(1.0,0.8,0.0, 0.9*tab_alpha*(1.0-black));
    for (int i=0; i<32; ++i) {
        if (i*10 > (((maxy-120)<?580)-330)) break;
        aGLPrintf(hAGL, minifont, 60, 320+(10*i), szTab[i]);
    }
    glDisable(GL_SCISSOR_TEST);


    glPopAttrib();
    return 0;
};

__export c_iface_def iface_def; 




