/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include "koule.hpp"
#include "../aLib/aStdEx/aStdEx.h"

int DrawSphere(float radius, int n1, int n2) {
    if (n1 < 2 || n2 < 2) return 0;
    
    float *a = (float*)alloc((n2+1)*sizeof(float));
    float *b = (float*)alloc((n1+1)*sizeof(float));

    float *tmp1 = (float*)alloc((n1+1)*sizeof(float)*2);
    float *tmp2 = (float*)alloc((n1+1)*sizeof(float)*2);
    
    if (!a || !b || !tmp1 || !tmp2) {
        free(a); free(b); free(tmp1); free(tmp2);
        return 0;
    }

    for (int i = 0; i < n2+1; ++i) a[i] = cos(PI*(float)i/(float)n2);
    for (int i = 0; i < n1+1; ++i) b[i] = 2.0f*PI*((float)i/(float)n1);
    
    float x, y, m, tmplast[2], last[2];

//    glBegin(GL_POINTS);
    glBegin(GL_QUADS);
//    glBegin(GL_LINES);
    for (int j = 0; j <= n2; ++j) {
        m = sqrt( 1.0f - a[j]*a[j] );
            
        for (int i = 0; i <= n1; ++i) 
        {
            x = m * sin( b[i] );
            y = m * cos( b[i] );
        
            if (j && i) { // render quad
                float tc[4] = {
                    (float)(n1-i)/(float)n1,
                    (float)(n1-(i+1))/(float)n1,
                    (float)(j-1)/(float)(n2),
                    (float)(j+0)/(float)(n2)
//                    (float)j/(float)n2,
//                    (float)(j+1)/(float)n2
                };
            
                glTexCoord2f( tc[1], tc[2] );
                glNormal3f( tmp1[2*i+0], tmp1[2*i+1], a[j-1] );
                glVertex3f( radius * tmp1[2*i+0], radius * tmp1[2*i+1], radius * a[j-1] );

                glTexCoord2f( tc[0], tc[2] );
                glNormal3f( tmp1[2*(i-1)+0], tmp1[2*(i-1)+1], a[j-1] );
                glVertex3f( radius * tmp1[2*(i-1)+0], radius * tmp1[2*(i-1)+1],
                            radius * a[j-1] );
            
                glTexCoord2f( tc[0], tc[3] );
                glNormal3f( last[0], last[1], a[j] );
                glVertex3f( radius * last[0], radius * last[1], radius * a[j] );

                glTexCoord2f( tc[1], tc[3] );
                glNormal3f( x, y, a[j] );
                glVertex3f( radius * x, radius * y, radius * a[j] );
            }

            tmp2[2*i+0] = x;
            tmp2[2*i+1] = y;
            
            last[0] = x;
            last[1] = y;
        }
        
        memcpy(tmp1, tmp2, (n1+1)*sizeof(float)*2);
    }
    glEnd();

    free(tmp2);
    free(tmp1);
    free(b);
    free(a);
    return 0;
};

