/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include <windows.h>
#include <fmod.h>

#include "../aGL/aGL.hpp"
#include "../aLib/random/random.hpp"
#include "../aLib/aMode/aMode.hpp"

#include "particle.hpp"
#include "scene_blank.hpp"
#include "scene_galaxy.hpp"
#include "scene_planets.hpp"
#include "scene_restart.hpp"

#include "warp.hpp" // effect

#include "iface_init.hpp"
#include "iface_def.hpp"

#define FPS_LATENCY 32

HINSTANCE hInst;
HWND hwnd;
char szAppName[ ] = "Space 2004 - Solar system";

__export int maxx, maxy;
__export int zmenit_pohled = false;

int CreateClass();
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain (HINSTANCE hInst,HINSTANCE hPrevInst,LPSTR lpszArg,int nStyle){
    MSG messages;
    ::hInst = hInst;
   
    if (!CreateClass()) return 0;
    
    maxx = GetSystemMetrics(SM_CXSCREEN);
    maxy = GetSystemMetrics(SM_CYSCREEN);

    hwnd = CreateWindowEx (0, szAppName, szAppName,
           WS_POPUP, 0,0, maxx, maxy,
           HWND_DESKTOP, NULL, hInst, NULL);

    ShowWindow (hwnd, nStyle);
    ShowCursor (FALSE);

    FSOUND_SetHWND(hwnd);
    FSOUND_SetOutput(FSOUND_OUTPUT_WINMM);
    FSOUND_Init(44100, 16, FSOUND_INIT_GLOBALFOCUS);

    bool Quit = false;
    while (!Quit){
        if (PeekMessage (&messages, NULL, 0, 0, PM_REMOVE))  {
            if (messages.message == WM_QUIT) {
                Quit = TRUE;
            } else {  
                TranslateMessage(&messages);
                DispatchMessage(&messages);
            }
        } else {
            InvalidateRect(hwnd, NULL, false);
        }
    }

/*    while (GetMessage (&messages, NULL, 0, 0)){
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }//*/

    ShowCursor (TRUE);
    
    FSOUND_Close();

    return messages.wParam;
}

int CreateClass() {
    WNDCLASSEX wincl;

    wincl.hInstance = hInst;
    wincl.lpszClassName = szAppName;
    wincl.lpfnWndProc = WindowProcedure;
    wincl.style = CS_DBLCLKS;
    wincl.cbSize = sizeof (WNDCLASSEX);
    wincl.hIcon = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hIconSm = LoadIcon (NULL, IDI_APPLICATION);
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = NULL;
    wincl.cbClsExtra = 0;
    wincl.cbWndExtra = 0;
    wincl.hbrBackground = (HBRUSH) CreateSolidBrush( GetSysColor( COLOR_3DFACE ) );

    if (!RegisterClassEx (&wincl)) return 0;
    return 1;
};

FSOUND_STREAM *amb1, *amb2;

__export aHANDLE hAGL;
__export aFrustum frustum;
__export bool keyboard[256];

__export aState scene;
__export aState effect;
__export aState iface;

__export aFontHANDLE minifont, font, sysfont, midfont, bigfont;
aBitmapGL bitmap;

LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam){
    switch (message){
        case WM_CREATE:
            SetFocus(hwnd);
            hAGL = aGLCreateContext(hwnd);
            hAGL->farplane = 6000000.0; // 6 miliard km
            hAGL->nearplane = 1.0; // thousand kilometer
            aGLSetClearColor(hAGL, 0);

            SetTimer(hwnd, 1, 50, NULL);
            font = aGLCreateFont( hAGL, 14,600,0,DEFAULT_CHARSET,
                                  DEFAULT_PITCH|FF_SWISS,"Lucida Console");
            sysfont = aGLCreateFont( hAGL, 9,400,0,DEFAULT_CHARSET,
                                     FIXED_PITCH|FF_SWISS,"Fixedsys");
            midfont = aGLCreateFont( hAGL, 24,400,0,DEFAULT_CHARSET,
                                     DEFAULT_PITCH|FF_SWISS,"Lucida Console");
            bigfont = aGLCreateFont( hAGL, 36,700,0,DEFAULT_CHARSET,
                                     DEFAULT_PITCH|FF_SWISS,"Lucida Console");
            minifont = aGLCreateFont( hAGL, 8,400,0,DEFAULT_CHARSET,
                                      DEFAULT_PITCH|FF_SWISS,"Lucida Console");
        return 0;

        case WM_SYSCOMMAND:
            if (wParam == SC_SCREENSAVE ) return FALSE;    
            if (wParam == SC_MONITORPOWER ) return FALSE;
        return DefWindowProc (hwnd, message, wParam, lParam); 
        
        case WM_TIMER: {
            KillTimer(hwnd, wParam);
            InitParticle();

            scene.Register(&scene_blank);
            scene.Register(&scene_galaxy);
            scene.Register(&scene_planets);
            scene.Register(&scene_restart);
            
            effect.Register(&scene_blank);
            effect.Register(&effect_warp);

            iface.Register(&scene_blank);
            iface.Register(&iface_init);
            iface.Register(&iface_def);
            
            amb1 = FSOUND_Stream_Open("data/ambient.ogg", FSOUND_LOOP_NORMAL, 0, 0);

            FSOUND_SetSFXMasterVolume(255);
            
            FSOUND_Stream_Play(0, amb1);
            FSOUND_SetVolume(0, 0);
            
            scene = 1; // TODO: tady 1
            effect = 0;
            iface = 1;
            
        } break;
        
        case WM_SIZE:
            aGLResize(hAGL,LOWORD(lParam),HIWORD(lParam));
            InvalidateRect(hwnd, NULL, false);
        break;
         
        case WM_DESTROY:
            aGLDeleteObject(hAGL,font);
            aGLDeleteObject(hAGL,sysfont);
            aGLDeleteObject(hAGL,minifont);
            aGLDeleteObject(hAGL,midfont);
            aGLDeleteObject(hAGL,bigfont);

            aGLDeleteContext(hwnd,hAGL);
            PostQuitMessage (0);
        break;
        
        case WM_KEYDOWN: keyboard[wParam] = true;  break;
        case WM_KEYUP:   keyboard[wParam] = false; break;
        
        case WM_CHAR: {
            if (wParam == VK_ESCAPE) PostMessage(hwnd, WM_CLOSE, 0, 0);
        } break;

        case WM_PAINT: {
            static int time = 0;
            int _t = GetTickCount();

            aGLBeginPaint(hAGL);
            aGL3D(hAGL);
            glPushAttrib(GL_ALL_ATTRIB_BITS);
            
                scene.SetTime(time);
                effect.SetTime(time);
                iface.SetTime(time);
    
                if (scene.loop() == AMODE_SIGNAL_ENDREQUESTED)
                    PostMessage(hwnd, WM_CLOSE, 0, 0);

            glPopAttrib();
            aGL2D(hAGL); // effects and interface are 2D
            glPushAttrib(GL_ALL_ATTRIB_BITS);

                if (effect.loop() == AMODE_SIGNAL_ENDREQUESTED)
                    PostMessage(hwnd, WM_CLOSE, 0, 0);

            glPopAttrib();
            glPushAttrib(GL_ALL_ATTRIB_BITS);

                if (iface.loop() == AMODE_SIGNAL_ENDREQUESTED)
                    PostMessage(hwnd, WM_CLOSE, 0, 0);

            glPopAttrib();
            aGL2D(hAGL);
            glPushAttrib(GL_ALL_ATTRIB_BITS);

                glLoadIdentity();
                glColor4f(1.0,1.0,1.0,1.0);
                if (!time) time = 1;
                aGLLighting(hAGL, false);
                glDisable(GL_TEXTURE_2D);
                
                if (iface == AMODE_ERROR_NOMODESELECTED) {
                    aGLPrintf(hAGL, font, 40,40, "Czech 3D Open Contest");
                    aGLPrintf(hAGL, font, 40,55, "Space 2004: Solar system");
                    aGLPrintf(hAGL, font, 40,85, "Loading, please wait ...");
                } else if (iface > 1) {
                    static int tlast[FPS_LATENCY];
                    static int tcur = 0;
                    
                    tlast[tcur++] = time?:1; if (tcur >= FPS_LATENCY) tcur = 0;
                    
                    float fps = 0.0f;
                    for (int i=0; i<FPS_LATENCY; ++i) fps += tlast[i];
                    fps /= (float)FPS_LATENCY;
                    
                    char aaa[256];
                    sprintf(aaa,"%3.1f / %3.1f",(fps)?1000.0/fps:1000.0, fps);
                    
                    if (iface_def.black < 0.1) {
                        glColor4f(0.0,0.0,0.0, 1.0);
                        aGLPrintf(hAGL, sysfont, maxx-380+2,63+2, aaa);
                        glColor4f(1.0,0.8,0.0, 0.9);
                        aGLPrintf(hAGL, sysfont, maxx-380+1,63+1, aaa);
                    }
                }

            glPopAttrib();

            aGLEndPaint(hwnd,hAGL);

            time = GetTickCount() - _t;
        } break;

          
        default: return DefWindowProc (hwnd, message, wParam, lParam);
    }
    return 0;
}


