/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include "particle.hpp"
#include "billboard.hpp"
#include "../aLib/random/random.hpp"

int particle_texture;

int InitParticle() {
    aBitmapGL bitmap;
    
    bitmap.load("data/star.aBitmap");
    particle_texture = bitmap.getOpenGLTexture();
    
    randomize();
};

int __fastcall BeginParticles() {
    glBindTexture(GL_TEXTURE_2D, particle_texture);
    BillboardSphericalBegin();
};

int __fastcall EndParticles() {
    BillboardSphericalEnd();
};

int __fastcall DrawParticle(float size) {
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0*size, -1.0*size, 0.0);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0*size,  1.0*size, 0.0);
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0*size,  1.0*size, 0.0);
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0*size, -1.0*size, 0.0);
    glEnd();
};

struct spray {
    VECTOR3D pos, vec;

    float life; // zivotnost castice
    float dist; // rozptyl
    float acc; // akcelerace, 0.x znamena deceleraci
    float size; // velikost castic
    
    float num;
    
    struct __part {
        bool en;
        float life;
        VECTOR3D pos, vec;
    } pt[1024];    
};

int StartSpray(VECTOR3D pos, VECTOR3D smer, float pow,
               float life, float dist, float acc, float size, float num) {
    spray* aaa = new spray;

    aaa->pos = pos;
    aaa->vec = smer;
        aaa->vec.x *= pow;
        aaa->vec.y *= pow;
        aaa->vec.z *= pow;
    aaa->life = life;
    aaa->dist = dist;
    aaa->acc = acc;
    aaa->size = size;
    aaa->num = num;
    
    for (int i=0; i<1024; ++i) aaa->pt[i].en = false;
    
    return (int)aaa;
};

// NOTE: This function does not work properly
int ComputeSpray(int handle, int t) {
    spray* spr = (spray*)handle;

    static float n = 0.0;
    n += spr->num * t * 0.001;
    
    for (int i=0; i<1024; ++i) {
        if (!spr->pt[i].en) {
            if (n > 1.0) {
                --n;
                spr->pt[i].en = true;
                spr->pt[i].pos = spr->pos;
                spr->pt[i].vec = spr->vec;
                spr->pt[i].life = 0.0;

                if (spr->dist < 0.01) {
                    spr->pt[i].vec.x = (1000.0 - random(2000))/1000.0 * spr->pt[i].vec.x;
                    spr->pt[i].vec.y = (1000.0 - random(2000))/1000.0 * spr->pt[i].vec.y;
                    spr->pt[i].vec.z = (1000.0 - random(2000))/1000.0 * spr->pt[i].vec.z;
                } else {
                    spr->pt[i].vec.x *= spr->dist;
                    spr->pt[i].vec.y *= spr->dist;
                    spr->pt[i].vec.z *= spr->dist;
                }
            }
            continue;
        }

        spr->pt[i].pos.x += 0.001*t*spr->pt[i].vec.x;
        spr->pt[i].pos.y += 0.001*t*spr->pt[i].vec.y;
        spr->pt[i].pos.z += 0.001*t*spr->pt[i].vec.z;
        
        for (int j=0; j<t; ++j) {
            spr->pt[i].vec.x *= spr->acc;
            spr->pt[i].vec.y *= spr->acc;
            spr->pt[i].vec.z *= spr->acc;
        }
        
        spr->pt[i].life += t/1000.0;
        
        if (spr->pt[i].life > spr->life) spr->pt[i].en = false;
    }

    return 0;
};

int DrawSpray(int handle) {
    spray* spr = (spray*)handle;
    float clr[4];
    
    BeginParticles();
    for (int i=0; i<1024; ++i) {
        if (!spr->pt[i].en) continue;
        
        glGetFloatv(GL_CURRENT_COLOR, clr);
        glPushMatrix();
            glTranslatef( spr->pt[i].pos.x, spr->pt[i].pos.y, spr->pt[i].pos.z ); 
            
            float mult = 1.0 - (float)spr->pt[i].life/(float)spr->life;
            glColor3f( clr[0] * mult, clr[1] * mult, clr[2] * mult );

            DrawParticle( spr->size );
        glPopMatrix();
        
        glColor3fv(clr);
    }    
    EndParticles();

    return 0;
};

int StopSpray(int handle) {
    spray* spr = (spray*)handle;
    free(spr);
    return 0;    
};



