/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include "../aLib/random/random.hpp"
#include "scene_planets.hpp"
#include "hvezda.hpp"
#include "galaxie.hpp"
#include "asteriod.hpp"

#include "slunce.hpp"

#include "merkur.hpp"
#include "venuse.hpp"
#include "zeme.hpp"
 #include "mesic.hpp"
#include "mars.hpp"
 #include "phobos.hpp"
 #include "deimos.hpp"

#include "jupiter.hpp"
 #include "jupiter_mesice.hpp"
#include "saturn.hpp"
 #include "saturn_mesice.hpp"
#include "uran.hpp"
 #include "uran_mesice.hpp"
#include "neptun.hpp"
 #include "triton.hpp"
#include "pluto.hpp"
 #include "charon.hpp"

#include "iface_def.hpp"
#include "vypis.hpp"

#include "scene_planets_data.hpp"

struct __planets_string {
    char text[64];
    float time;
    bool set;
} planets_string[16] = {
    { "Pluto - Charon",      8.0, false },
    { "Neptune",            28.0, false },
    { "Uranus",             50.0, false },
    { "Saturn",             90.0, false },
    { "Jupiter",           145.0, false },
    { "Mars",              175.0, false },
    { "The Moon",          210.0, false },
    { "Earth",             235.0, false },
    { "Venus",             270.0, false },
    { "Mercury",           300.0, false },
    { "The Sun",           320.0, false },

}, top_part_string[8] = {
    { "Kuiper Belt",         4.0, false },
    { "Outer Sol System",   20.0, false },
    { "Asteroid Belt",     166.0, false },
    { "Inner Sol System",  180.0, false },
    { "Entering Corona",   330.0, false },
};

aPath planets_desc_alpha;

struct __planets_descs {
    char text[1024];
    float time;
    bool set;
} planets_desc[16] = {
    {   "Pluto:\n \n"
        "orbit:    5,913,520,000 km (39.5 AU)\n"
        "diameter: 2274 km\n"
        "mass:     1.27e22 kg\n \n"
        "Charon:\n \n"
        "orbit: 19,640 km from Pluto\ndiameter: 1172 km\nmass: 1.90e21 kg\n \n \n"
        "Pluto's equator is at almost right angles\n"
        "to the plane of its orbit. The surface\n"
        "temperature on Pluto varies between about\n"
        "-235 and -210 C (38 to 63 K).\n"
        "Pluto is the farthest planet from the Sun.\n"
        "Some consider it to be the largest of the\n"
        "Kuiper Belt objects.",      8.0, false },

    {   "Neptune\n \n"
        "orbit:    4,504,000,000 km (30.06 AU)\n"
        "diameter: 49,532 km (equatorial)\n"
        "mass:     1.0247e26 kg\n \n"
        "Neptune was first observed by Galle and\n"
        "d'Arrest on 1846 Sept 23.\n"
        "Neptune's blue color is largely the result\n"
        "of absorption of red light by methane in\n"
        "the atmosphere but there is some additional\n"
        "as-yet-unidentified chromophore which gives\n"
        "the clouds their rich blue tint.\n \n"
        "Neptune also has rings. Like Uranus and\n"
        "Jupiter, Neptune's rings are very dark but\n"
        "their composition is unknown.\n \n"
        "The largest satelites:\n \n"
        "Proteus - distance: 118  - radius: 209\n"
        "Triton  - distance: 355  - radius: 1350\n"
        "Nereid  - distance: 5509 - radius: 170\n",     30.0, false },

    {   "Uranus\n \n"
        "orbit:    2,870,990,000 km (19.218 AU)\n"
        "diameter: 51,118 km (equatorial)\n"
        "mass:     8.683e25 kg\n \n"
        "The earliest recorded sighting was in 1690\n"
        "when John Flamsteed cataloged it as 34Tauri\n"
        "Uranus is composed primarily of rock and\n"
        "various ices\n \n"
        "Uranus' atmosphere is about 83%% hydrogen,\n"
        "15%% helium and 2%% methane. Uranus' blue color\n"
        "is the result of absorption of red light by\n"
        "methane in the upper atmosphere.\n \n"
        "Uranus has rings and 21 named moons and six\n"
        "unnamed ones.\n \n"
        "The largest satelites:\n \n"
        "Miranda - distance: 130 - radius: 236\n"
        "Ariel   - distance: 191 - radius: 579\n"
        "Umbriel - distance: 266 - radius: 585\n"
        "Titania - distance: 436 - radius: 789\n"
        "Oberon  - distance: 583 - radius: 761\n"
        , 56.0, false },

    {   "Saturn\n \n"
        "orbit:    1,429,400,000 km (9.54 AU)\n"
        "diameter: 120,536 km (equatorial)\n"
        "mass:     5.68e26 kg\n \n"
        "Saturn's rings, unlike the rings of the other\n"
        "planets, are very bright. When it is in the\n"
        "nighttime sky, Saturn is easily visible to\n"
        "the unaided eye.\n \n"
        "Saturn is visibly flattened (oblate) when\n"
        "viewed through a small telescope; its equatorial\n"
        "and polar diameters vary by almost 10%%\n"
        "(120,536 km vs. 108,728 km). Saturn has 30\n"
        "named satellites plus one discovered in 2003\n"
        "and as yet unnamed.\n \n"
        "The largest satelites:\n \n"
        "Mimas     - distance: 186  - radius: 196\n"
        "Enceladus - distance: 238  - radius: 260\n"
        "Tethys    - distance: 295  - radius: 530\n"
        "Dione     - distance: 377  - radius: 560\n"
        "Rhea      - distance: 527  - radius: 765\n"
        "Titan     - distance: 1222 - radius: 2575\n"
        "Iapetus   - distance: 3561 - radius: 730\n"
        ,100.0, false },

    {   "Jupiter\n \n"
        "orbit:    778,330,000 km (5.20 AU)\n"
        "diameter: 142,984 km (equatorial)\n"
        "mass:     1.900e27 kg\n \n"
        "Jupiter is the fifth planet from the Sun\n"
        "and by far the largest. The gas planets do\n"
        "not have solid surfaces, their gaseous\n"
        "material simply gets denser with depth\n"
        "(the radii and diameters quoted for the\n"
        "planets are for levels corresponding to a\n"
        "pressure of 1 atmosphere). Jupiter is about\n"
        "90%% hydrogen and 10%% helium with traces\n"
        "of methane, water, ammonia and \"rock\".\n \n"
        "The Great Red Spot (GRS) has been seen by\n"
        "Earthly observers for more than 300 years.\n \n"
        "Jupiter has 63 known satellites.\n"
        "The largest satellites:\n \n"
        "Io       - distance: 422  - radius: 1815\n"
        "Europa   - distance: 671  - radius: 1569\n"
        "Ganymede - distance: 1070 - radius: 2631\n"
        "Callisto - distance: 1883 - radius: 2400\n"
        ,151.0, false },

    {   "Mars\n \n"
        "orbit:    227,940,000 km (1.52 AU)\n"
        "diameter: 6,794 km\n"
        "mass:     6.4219e23 kg\n \n"
        "The planet probably got this name due to\n"
        "its red color; Mars is sometimes referred\n"
        "to as the Red Planet. Mars' orbit is\n"
        "significantly elliptical. While the average\n"
        "temperature on Mars is about 218 K (-55 C,\n"
        "-67 F), Martian surface temperatures range\n"
        "widely from as little as 140 K (-133 C, \n"
        "-207 F) at the winter pole to almost 300 K\n"
        "(27 C, 80 F) on the day side during summer.\n \n"
        "Mars has two tiny satellites which orbit very\n"
        "close to the martian surface:\n \n"
        "Phobos - distance: 9  - radius: 11\n"
        "Deimos - distance: 23 - radius: 6\n"
        ,179.0, false },

    {   "Earth\n \n"
        "orbit:    149,600,000 km (1.00 AU)\n"
        "diameter: 12,756.3 km\n"
        "mass:     5.972e24 kg\n \n"
        "The core is probably composed mostly of iron\n"
        "(or nickel/iron) though it is possible that\n"
        "some lighter elements may be  present, too.\n"
        "Temperatures at the center of the core may be\n"
        "as high as 7500 K, hotter than the surface of\n"
        "the Sun. The Earth is the densest major body\n"
        "in the solar system. The Earth's atmosphere is\n"
        "77%% nitrogen, 21%% oxygen, with traces of\n"
        "argon, carbon dioxide and water.\n \n"
        "Earth has only one natural satellite, the\n"
        "Moon. But Asteroids 3753 Cruithne and 2002\n"
        "AA29 have complicated orbital relationships\n"
        "withthe Earth.\n \n"
        "The Moon - distance: 384 - radius: 1738"
        ,235.0, false },

    {   "Venus\n \n"
        "orbit:    108,200,000 km (0.72 AU)\n"
        "diameter: 12,103.6 km\n"
        "mass:     4.869e24 kg\n \n"
        "Venus' orbit is the most nearly circular of\n"
        "that of any planet, with an eccentricity of\n"
        "less than 1%%. Venus has been known since\n"
        "prehistoric times. Venus' rotation is somewhat\n"
        "unusual in that it is both very slow (243\n"
        "Earth days per Venus day, slightly longer than\n"
        "Venus' year). Venus is only slightly smaller\n"
        "than Earth (95%% of Earth's diameter, 80%% of\n"
        "Earth's mass). Venus has no magnetic field,\n"
        "perhaps because of its slow rotation. The\n"
        "greenhouse effect is much stronger on Venus\n"
        "than Earth because of Venus' dense carbon\n"
        "dioxide atmosphere\n \n"
        "Venus has no satellites.\n"
        ,270.0, false },

    {   "Mercury\n \n"
        "orbit:    57,910,000 km (0.38 AU)\n"
        "diameter: 4,880 km\n"
        "mass:     3.30e23 kg\n"
        "Mercury has been known since at least the\n"
        "time of the Sumerians (3rd millennium BC).\n"
        "Mercury's orbit is highly eccentric; at\n"
        "perihelion it is only 46 million km from\n"
        "the Sun but at aphelion it is 70 million.\n"
        "Mercury rotates three times int two of its\n"
        "years. Temperature variations on Mercury\n"
        "are the most extreme in the solar system\n"
        "ranging from 90 K to 700 K. The temperature\n"
        "on Venus is slightly hotter but very stable.\n \n"
        "Mercury is often visible with binoculars or\n"
        "even the unaided eye, but it is always very\n"
        "near the Sun and difficult to see in the\n"
        "twilight sky. \n \n"
        "Mercury has no known satellites.\n"
        ,300.0, false },

    {   "The Sun\n \n"
        "diameter:    1,390,000 km.\n"
        "mass:        1.989e30 kg\n"
        "temperature: 5800 K (surface)\n"
        "             15,600,000 K (core)\n \n"
        "It is often said that the Sun is an\n"
        "\"ordinary\" star. That's true in the sense\n"
        "that there are many others similar to it.\n"
        "But there are many more smaller stars than\n"
        "larger ones; the Sun is in the top 10%% by\n"
        "mass. The Sun is, at present, about 70%%\n"
        "hydrogen and 28%% helium by mass everything\n"
        "else (\"metals\") amounts to less than 2%%.\n"
        "The Sun's energy output (3.86e33 ergs/second\n"
        "or 386 billion billion megawatts) is\n"
        "produced by nuclear fusion reactions. Each\n"
        "second about 700,000,000 tons of hydrogen\n"
        "are converted to about 695,000,000 tons of\n"
        "helium and 5,000,000 tons (=3.86e33 ergs) of\n"
        "energy in the form of gamma rays.\n"
        "There are nine planets and a large number of\n"
        "smaller objects orbiting the Sun. (Exactly\n"
        "which bodies should be classified as planets\n"
        "and which as \"smaller objects\" has been the\n"
        "source of some controversy, but in the end it\n"
        "is really only a matter of definition.)\n"
        
        ,320.0, false },
};

aPath pad_alpha[4];
int pad_tx[4];

struct __pad_images {
    char text[64];
    float time;
    bool set;
} pad_img[4][8] = {
    { // pad[0]
        { "Charon",  10.0, false },
        { "Triton",  32.0, false },
        { "Ariel",   58.0, false },
        { "Mimas",  102.0, false },
        { "IO",     153.0, false },
        { "Phobos", 181.0, false },
        { "The Moon",237.0,false },
    },
    { // pad[1]
        { "Umbriel", 58.0, false },
        { "Rhea",   102.0, false },
        { "Europa", 153.0, false },
        { "Deimos", 181.0, false },
    },
    { // pad[2]
        { "Titania",  58.0,false },
        { "Titan",   102.0,false },
        { "Ganymede",153.0,false },
    },
    { // pad[3]
        { "Oberon",  58.0, false },
        { "Callisto",153.0,false },
    }    
};



struct c_scene_planets_data {
    int t;

    DVECTOR3D camera_pos, camera_target, camera_up;
    
    // pohyb kamery
    aPath apCameraPos;
    aPath apCameraTarget;
    aPath apCameraUp;
    aPath apCameraType; // prvn parametr 0.0 - sinus, cokoliv jineho linear
    
    double time; // time in pos
    
    c_scene_planets_data() {
        FillPaths( &apCameraType, &apCameraPos, &apCameraTarget, &apCameraUp );
    
        reset();
        return;
    };
    
    int reset() {
        t = 0;
        time = 0.0;
        return 0;
    };
} scene_planets_data;

int c_scene_planets::init() {
    aBitmapGL bitmap;

    InitVypis(); 

    bitmap.load("data/starbox.aBitmap");
    InitStars(&bitmap);

    bitmap.load("data/milkyway.aBitmap");
    InitGalaxy(&bitmap);
    
    InitAsteroidy();
    
    InitSlunce();
    InitMerkur();
    InitVenuse();
    InitZeme();
        InitMesic();
    InitMars();
        InitPhobos();
        InitDeimos();
    InitJupiter();
        InitJupiterMesice();
    InitSaturn();
        InitSaturnMesice();
    InitUran();
        InitUranMesice();
    InitNeptun();
        InitTriton();
    InitPluto();
        InitCharon();
    
    data = &scene_planets_data;
    
    planets_desc_alpha.addp(  0.0, 0.0);
    
    planets_desc_alpha.addp(  8.0, 0.0); // pluto
    planets_desc_alpha.addp( 10.0, 1.0);
    planets_desc_alpha.addp( 20.0, 1.0);
    planets_desc_alpha.addp( 22.0, 0.0);

    planets_desc_alpha.addp( 30.0, 0.0); // neptun
    planets_desc_alpha.addp( 32.0, 1.0);
    planets_desc_alpha.addp( 42.0, 1.0);
    planets_desc_alpha.addp( 44.0, 0.0);

    planets_desc_alpha.addp( 56.0, 0.0); // uran
    planets_desc_alpha.addp( 58.0, 1.0);
    planets_desc_alpha.addp( 68.0, 1.0);
    planets_desc_alpha.addp( 70.0, 0.0);

    planets_desc_alpha.addp(100.0, 0.0); // saturn
    planets_desc_alpha.addp(102.0, 1.0);
    planets_desc_alpha.addp(112.0, 1.0);
    planets_desc_alpha.addp(114.0, 0.0);

    planets_desc_alpha.addp(151.0, 0.0); // jupiter
    planets_desc_alpha.addp(153.0, 1.0);
    planets_desc_alpha.addp(163.0, 1.0);
    planets_desc_alpha.addp(165.0, 0.0);

    planets_desc_alpha.addp(179.0, 0.0); // mars
    planets_desc_alpha.addp(181.0, 1.0);
    planets_desc_alpha.addp(189.0, 1.0);
    planets_desc_alpha.addp(191.0, 0.0);

    planets_desc_alpha.addp(235.0, 0.0); // zeme
    planets_desc_alpha.addp(237.0, 1.0);
    planets_desc_alpha.addp(250.0, 1.0);
    planets_desc_alpha.addp(252.0, 0.0);

    planets_desc_alpha.addp(270.0, 0.0); // venuse
    planets_desc_alpha.addp(272.0, 1.0);
    planets_desc_alpha.addp(282.0, 1.0);
    planets_desc_alpha.addp(284.0, 0.0);

    planets_desc_alpha.addp(300.0, 0.0); // merkur
    planets_desc_alpha.addp(302.0, 1.0);
    planets_desc_alpha.addp(312.0, 1.0);
    planets_desc_alpha.addp(314.0, 0.0);

    planets_desc_alpha.addp(320.0, 0.0); // slunicko
    planets_desc_alpha.addp(322.0, 1.0);
    planets_desc_alpha.addp(337.0, 1.0);
    planets_desc_alpha.addp(339.0, 0.0);

    for (int i=0; i<4; ++i) pad_alpha[i].reset();
    pad_alpha[0].addp( 10.0, 0.0); // charon
    pad_alpha[0].addp( 12.0, 1.0);
    pad_alpha[0].addp( 18.0, 1.0);
    pad_alpha[0].addp( 20.0, 0.0);
    
    pad_alpha[0].addp( 32.0, 0.0); // triton
    pad_alpha[0].addp( 34.0, 1.0);
    pad_alpha[0].addp( 40.0, 1.0);
    pad_alpha[0].addp( 42.0, 0.0);

    pad_alpha[0].addp( 58.0, 0.0); // uran ariel
    pad_alpha[0].addp( 60.0, 1.0);
    pad_alpha[0].addp( 66.0, 1.0);
    pad_alpha[0].addp( 68.0, 0.0);

    pad_alpha[0].addp(102.0, 0.0); // saturn - mimas
    pad_alpha[0].addp(104.0, 1.0);
    pad_alpha[0].addp(110.0, 1.0);
    pad_alpha[0].addp(112.0, 0.0);

    pad_alpha[0].addp(153.0, 0.0); // jupiter - io
    pad_alpha[0].addp(155.0, 1.0);
    pad_alpha[0].addp(161.0, 1.0);
    pad_alpha[0].addp(163.0, 0.0);

    pad_alpha[0].addp(181.0, 0.0); // mars - phobos
    pad_alpha[0].addp(183.0, 1.0);
    pad_alpha[0].addp(191.0, 1.0);
    pad_alpha[0].addp(193.0, 0.0);

    pad_alpha[0].addp(237.0, 0.0); // zeme - mesic
    pad_alpha[0].addp(239.0, 1.0);
    pad_alpha[0].addp(250.0, 1.0);
    pad_alpha[0].addp(252.0, 0.0);


    pad_alpha[1].addp( 59.0, 0.0); // uran umbirel
    pad_alpha[1].addp( 61.0, 1.0);
    pad_alpha[1].addp( 67.0, 1.0);
    pad_alpha[1].addp( 69.0, 0.0);

    pad_alpha[1].addp(103.0, 0.0); // saturn - rhea
    pad_alpha[1].addp(105.0, 1.0);
    pad_alpha[1].addp(111.0, 1.0);
    pad_alpha[1].addp(113.0, 0.0);

    pad_alpha[1].addp(154.0, 0.0); // jupiter - europa
    pad_alpha[1].addp(156.0, 1.0);
    pad_alpha[1].addp(162.0, 1.0);
    pad_alpha[1].addp(164.0, 0.0);

    pad_alpha[1].addp(182.0, 0.0); // mars - deimos
    pad_alpha[1].addp(184.0, 1.0);
    pad_alpha[1].addp(192.0, 1.0);
    pad_alpha[1].addp(194.0, 0.0);



    pad_alpha[2].addp( 60.0, 0.0); // uran titania
    pad_alpha[2].addp( 62.0, 1.0);
    pad_alpha[2].addp( 68.0, 1.0);
    pad_alpha[2].addp( 70.0, 0.0);

    pad_alpha[2].addp(104.0, 0.0); // saturn - titan
    pad_alpha[2].addp(106.0, 1.0);
    pad_alpha[2].addp(112.0, 1.0);
    pad_alpha[2].addp(114.0, 0.0);

    pad_alpha[2].addp(155.0, 0.0); // jupiter - ganymede
    pad_alpha[2].addp(157.0, 1.0);
    pad_alpha[2].addp(163.0, 1.0);
    pad_alpha[2].addp(165.0, 0.0);


    pad_alpha[3].addp( 61.0, 0.0); // uran oberon
    pad_alpha[3].addp( 63.0, 1.0);
    pad_alpha[3].addp( 69.0, 1.0);
    pad_alpha[3].addp( 71.0, 0.0);

    pad_alpha[3].addp(156.0, 0.0); // jupiter - callisto
    pad_alpha[3].addp(158.0, 1.0);
    pad_alpha[3].addp(164.0, 1.0);
    pad_alpha[3].addp(166.0, 0.0);



    return 0;
};

int c_scene_planets::entry() {
    c_scene_planets_data *data = (c_scene_planets_data*)this->data;
    data->reset();

    for (int i=0; i<16; ++i) planets_string[i].set = false;
    for (int i=0; i<8; ++i) top_part_string[i].set = false;
    for (int i=0; i<16; ++i) planets_desc[i].set = false;
    for (int i=0; i<4; ++i) for (int j=0; j<8; ++j) pad_img[i][j].set = false;
    for (int i=0; i<4; ++i) pad_tx[i] = 0;

/*
    selektor[0].length = 9.0;
    strcpy(selektor[0].name, "Hoessce System");
    selektor[0].apAlpha.addp(0.0, 0.0);
    selektor[0].apAlpha.addp(1.0, 1.0);
    selektor[0].apAlpha.addp(8.0, 1.0);
    selektor[0].apAlpha.addp(9.0, 0.0);
    selektor[0].apPos.addp(0.0,-0.50,-0.50);
    selektor[0].apPos.addp(1.0,-0.48,-0.48);
    selektor[0].apPos.addp(2.0,-0.05, 0.00);
    selektor[0].apPos.addp(8.0, 0.00,-0.01);
    selektor[0].apPos.addp(9.0, 0.20,-0.20);
*/

    // TODO: REMOVE ON RELEASE
//    data->time = 0.0;
    
    return 0;
};

int c_scene_planets::close() {
    return 0;
};

int c_scene_planets::evaluate(int time) {
    c_scene_planets_data *data = (c_scene_planets_data*)this->data;

    if (keyboard[VK_LEFT]) data->time -= time*0.1;
    if (keyboard[VK_RIGHT]) data->time += time*0.1;

    data->t = time;
    data->time += (double)data->t * 0.001;
    
    if (data->apCameraType.getpre(data->time) > 0.01) {
        data->apCameraPos.get( data->time, &data->camera_pos.x,
                               &data->camera_pos.y, &data->camera_pos.z);
        data->apCameraTarget.get( data->time, &data->camera_target.x,
                                  &data->camera_target.y, &data->camera_target.z);
        data->apCameraUp.get( data->time, &data->camera_up.x,
                              &data->camera_up.y, &data->camera_up.z);
    } else {
        data->apCameraPos.getsin( data->time, &data->camera_pos.x,
                                  &data->camera_pos.y, &data->camera_pos.z);
        data->apCameraTarget.getsin( data->time, &data->camera_target.x,
                                     &data->camera_target.y, &data->camera_target.z);
        data->apCameraUp.getsin( data->time, &data->camera_up.x,
                                 &data->camera_up.y, &data->camera_up.z);
    }
                          
    iface_def.right_x = data->camera_pos.x;
    iface_def.right_y = data->camera_pos.z;
    iface_def.right_z = data->camera_pos.y;
    
    // strings - dole
    for (int i=0; i<16; ++i) {
        if (planets_string[i].set) continue;
        if (!planets_string[i].text[0]) continue;

        if (planets_string[i].time < data->time) {
            planets_string[i].set = true;
            Dolu(planets_string[i].text);
        }
    }  
    for (int i=0; i<8; ++i) {
        if (top_part_string[i].set) continue;
        if (!top_part_string[i].text[0]) continue;

        if (top_part_string[i].time < data->time) {
            top_part_string[i].set = true;
            Nahoru(top_part_string[i].text);
        }
    }  
    for (int i=0; i<16; ++i) {
        if (planets_desc[i].set) continue;
        if (!planets_desc[i].text[0]) continue;

        if (planets_desc[i].time < data->time) {
            planets_desc[i].set = true;
            Tabulka(planets_desc[i].text);
        }
    }  

    for (int i=0; i<4; ++i) {
        for (int j=0; j<8; ++j) {
            if (pad_img[i][j].set) continue;
            if (!pad_img[i][j].text[0]) continue;

            if (pad_img[i][j].time < data->time) {
                pad_img[i][j].set = true;
                pad_tx[i]++;
                
                strcpy( iface_def.pad[i].sz, pad_img[i][j].text );
            }
        }

        iface_def.pad[i].alpha = pad_alpha[i].get( data->time );
    }  
    
    
    
    // animacka
    if (data->time > 365.0) {
        iface_def.black = (1.0 - ((375.0 - data->time)/10.0));
        if (iface_def.black > 1.0){
            iface_def.black = 1.0;
            
            if (effect == 0) effect = 1;
            if (zmenit_pohled) {
                zmenit_pohled = false;
                scene = 3; // outro
            }
        }
    }
    if (data->time > 350.0) {
        iface_def.progress = 1.0 - ((370.0 - data->time)/20.0);
        if (iface_def.progress > 1.0)
            iface_def.progress = 1.0;

        if (data->time > 355.0)
            iface_def.progress_alpha = 1.0;
        else
            iface_def.progress_alpha = 1.0 - ((355.0 - data->time) / 5.0);
    } else {
        iface_def.progress_alpha = 0.0;
        iface_def.progress = 0.0;
    }
    
    iface_def.tab_alpha = planets_desc_alpha.get(data->time);
    
    return 0;
};

int c_scene_planets::render() {
    c_scene_planets_data *data = (c_scene_planets_data*)this->data;
    
    // vykreslit texturu iface_def.tab_texture
    glEnable(GL_TEXTURE_2D);
    glPushAttrib(GL_ALL_ATTRIB_BITS);
        glViewport(0,0,128,128);
        
        glLoadIdentity();
        aGLLighting(hAGL, true);
        aGLUseLight(hAGL, 0, true);
        aGLOmniLight(hAGL, 0, 0.0, 0.0, 0.0);
        aGLLightColor(hAGL, 0, 0.1,0.1,0.1, 1.0, 1.0, 1.0, 1.0,1.0,1.0 );
        aGLMaterial(hAGL, 0.1,0.1,0.1,0.0, 1.0,1.0,1.0,1.0,
                          0.0,0.0,0.0,0.0, 0.0,0.0,0.0,0.0, 64);
        glDisable(GL_COLOR_MATERIAL);  

        if (iface_def.tab_alpha > 0.0) {
            glLoadIdentity();
            if (data->time > 320.0) {
                glTranslatef(0.0,0.0,-2000.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawSlunce (1.0, 0, false );
            } else if (data->time > 300.0) {
                glTranslatef(0.0,0.0,-7.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawMerkur (1.0, 0 );
            } else if (data->time > 270.0) {
                glTranslatef(0.0,0.0,-14.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawVenuse (1.0, 0 );
            } else if (data->time > 235.0) {
                glTranslatef(0.0,0.0,-14.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                glRotatef(90.0, 0.0, 1.0, 0.0);
                DrawZeme (1.0, 0 );
            } else if (data->time > 179.0) {
                glTranslatef(0.0,0.0,-12.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawMars (1.0, 0 );
            } else if (data->time > 151.0) {
                glTranslatef(0.0,0.0,-200.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawJupiter (1.0, 0 );
            } else if (data->time > 100.0) {
                glTranslatef(0.0,0.0,-300.0);
                glRotatef( 5.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawSaturn (1.0, 0 );
            } else if (data->time > 56.0) {
                glTranslatef(0.0,0.0,-100.0);
                glRotatef(20.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawUran (1.0, 0 );
            } else if (data->time > 30.0) {
                glTranslatef(0.0,0.0,-70.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawNeptun (1.0, 0 );
            } else if (data->time > 0.0) {
                glTranslatef(0.0,0.0,-3.0);
                glRotatef(30.0, 1.0, 0.0, 0.0);
                glRotatef(30.0, 0.0, 0.0, 1.0);
                DrawPluto (1.0, 0 );
            }
            
            glBindTexture(GL_TEXTURE_2D, iface_def.tab_texture);
            glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 0, 0, 128, 128, 0);
        }

        // prave texturky mesicu 
        
        for (int i=0; i<4; ++i) 
            if (iface_def.pad[i].alpha > 0.0) {
                glLoadIdentity();
                glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
                
                switch(i) {
                    case 0:
                        switch(pad_tx[i]) {
                            case 1:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawCharon (0.0, 0 );
                            break;
                            case 2:
                                glTranslatef(0.0,0.0,-5.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawTriton (0.0, 0 );
                            break;
                            case 3:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawUranMesic (i, 0.0, 0 );
                            break;
                            case 4:
                                glTranslatef(0.0,0.0,-1.3);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                glRotatef(180.0,0.0, 1.0, 0.0);
                                DrawSaturnMesic (i, 0.0, 0 );
                            break;
                            case 5:
                                glTranslatef(0.0,0.0,-7.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawJupiterMesic (i, 0.0, 0 );
                            break;
                            case 6:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                glScalef(50.0,50.0,50.0);
                                glEnable(GL_NORMALIZE);
                                DrawPhobos (0.0, 0 );
                                glDisable(GL_NORMALIZE);
                            break;
                            case 7:
                                glTranslatef(0.0,0.0,-9.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawMesic (0.0, 0 );
                            break;
                        };
                    break;
                    case 1:
                        switch(pad_tx[i]) {
                            case 1:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawUranMesic (1, 0.0, 0 );
                            break;
                            case 2:
                                glTranslatef(0.0,0.0,-3.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawSaturnMesic (i, 0.0, 0 );
                            break;
                            case 3:
                                glTranslatef(0.0,0.0,-5.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawJupiterMesic (i, 0.0, 0 );
                            break;
                            case 4:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                glScalef(50.0,50.0,50.0);
                                glEnable(GL_NORMALIZE);
                                DrawDeimos (0.0, 0 );
                                glDisable(GL_NORMALIZE);
                            break;
                        };
                    break;
                    case 2:
                        switch(pad_tx[i]) {
                            case 1:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawUranMesic (2, 0.0, 0 );
                            break;
                            case 2:
                                glTranslatef(0.0,0.0,-7.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawSaturnMesic (i, 0.0, 0 );
                            break;
                            case 3:
                                glTranslatef(0.0,0.0,-7.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawJupiterMesic (i, 0.0, 0 );
                            break;
                        };
                    break;
                    case 3:
                        switch(pad_tx[i]) {
                            case 1:
                                glTranslatef(0.0,0.0,-2.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawUranMesic (3, 0.0, 0 );
                            break;
                            case 2:
                                glTranslatef(0.0,0.0,-7.0);
                                glRotatef( 5.0, 1.0, 0.0, 0.0);
                                glRotatef(30.0, 0.0, 0.0, 1.0);
                                DrawJupiterMesic (i, 0.0, 0 );
                            break;
                        };
                    break;
                }
    
                glBindTexture(GL_TEXTURE_2D, iface_def.pad[i].texture);
                glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 0, 0, 128, 128, 0);
            }
        
    glPopAttrib();
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    
    // a konen 3D scnu
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
    glDisable(GL_TEXTURE_2D);

    glLoadIdentity();
    gluLookAt( data->camera_pos.x, data->camera_pos.y, data->camera_pos.z,
               data->camera_target.x, data->camera_target.y, data->camera_target.z, 
               data->camera_up.x, data->camera_up.y, data->camera_up.z );
    aGLLighting(hAGL, true);
    aGLUseLight(hAGL, 0, true);
    aGLOmniLight(hAGL, 0, 0.0, 0.0, 0.0);
    aGLLightColor(hAGL, 0, 0.1,0.1,0.1, 1.0, 1.0, 1.0, 1.0,1.0,1.0 );
    aGLMaterial(hAGL, 0.1,0.1,0.1,0.0, 1.0,1.0,1.0,1.0,
                      0.0,0.0,0.0,0.0, 0.0,0.0,0.0,0.0, 64);
    glDisable(GL_COLOR_MATERIAL);  

    glLoadIdentity();
    gluLookAt( 0,0,0,
               data->camera_target.x - data->camera_pos.x,
               data->camera_target.y - data->camera_pos.y,
               data->camera_target.z - data->camera_pos.z, 
               data->camera_up.x, data->camera_up.y, data->camera_up.z );
    frustum.CalculateFrustum();
    PaintStars();

    glRotatef(-50.0,0.0,0.0,1.0);
    frustum.CalculateFrustum();
    DrawGalaxy();


    glFrontFace(GL_CW);
    glEnable(GL_DEPTH_TEST);

    glLoadIdentity();
    gluLookAt( data->camera_pos.x, data->camera_pos.y, data->camera_pos.z,
               data->camera_target.x, data->camera_target.y, data->camera_target.z, 
               data->camera_up.x, data->camera_up.y, data->camera_up.z );
    frustum.CalculateFrustum();

    // NO CULL FACE FOR SUN
    if (frustum.SphereInFrustum(0,0,0,4000.0))
        DrawSlunce(PointDistance((DVECTOR3D){0,0,0}, data->camera_pos), data->t, true);

    glEnable(GL_CULL_FACE); 
   
    if (frustum.SphereInFrustum(planet_pos[0].x, planet_pos[0].y,
                                planet_pos[0].z, 2.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[0].x, planet_pos[0].y, planet_pos[0].z);
            DrawPluto (PointDistance(planet_pos[0], data->camera_pos), data->t );
        glPopMatrix();
    }
    if (frustum.SphereInFrustum(planet_pos[1].x, planet_pos[1].y,
                                planet_pos[1].z, 1.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[1].x, planet_pos[1].y, planet_pos[1].z);
            DrawCharon(PointDistance(planet_pos[1], data->camera_pos), data->t );
        glPopMatrix();
    }

    // neptun
    if (frustum.SphereInFrustum(planet_pos[3].x, planet_pos[3].y,
                                planet_pos[3].z, 2.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[3].x, planet_pos[3].y, planet_pos[3].z);
            glRotatef(30.0,1.0,1.0,0.0);
            DrawTriton(PointDistance(planet_pos[3], data->camera_pos), data->t );
        glPopMatrix();
    }
    if (frustum.SphereInFrustum(planet_pos[2].x, planet_pos[2].y,
                                planet_pos[2].z, 100.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[2].x, planet_pos[2].y, planet_pos[2].z);
            glRotatef(-20.0,1.0,1.0,0.0);
            DrawNeptun(PointDistance(planet_pos[2], data->camera_pos), data->t );
        glPopMatrix();
    }

    for (int i=0; i<4; ++i) {
        if (frustum.SphereInFrustum(planet_pos[5+i].x, planet_pos[5+i].y,
                                    planet_pos[5+i].z, 1.0)) {
          glPushMatrix(); 
            glTranslated(planet_pos[5+i].x, planet_pos[5+i].y, planet_pos[5+i].z);
            DrawUranMesic(i,PointDistance(planet_pos[5+i], data->camera_pos), data->t);
          glPopMatrix();
        }
    }

    if (frustum.SphereInFrustum(planet_pos[4].x, planet_pos[4].y,
                                planet_pos[4].z, 60.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[4].x, planet_pos[4].y, planet_pos[4].z);
            glRotatef(30.0,-1.0,-1.0,0.0);
            DrawUran (PointDistance(planet_pos[4], data->camera_pos), data->t );
        glPopMatrix();
    }

    for (int i=0; i<3; ++i) {
        if (frustum.SphereInFrustum(planet_pos[10+i].x, planet_pos[10+i].y,
                                    planet_pos[10+i].z, 3.0)) {
          glPushMatrix(); 
            glTranslated(planet_pos[10+i].x, planet_pos[10+i].y, planet_pos[10+i].z);
            DrawSaturnMesic(i,PointDistance(planet_pos[10+i], data->camera_pos),data->t);
          glPopMatrix();
        }
    }

    if (frustum.SphereInFrustum(planet_pos[9].x, planet_pos[9].y,
                                planet_pos[9].z, 200.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[9].x, planet_pos[9].y, planet_pos[9].z);
            glRotatef(-20.0,1.0,1.0,0.0);
            DrawSaturn (PointDistance(planet_pos[9], data->camera_pos), data->t );
        glPopMatrix();
    }

    for (int i=0; i<4; ++i) {
        if (frustum.SphereInFrustum(planet_pos[14+i].x, planet_pos[14+i].y,
                                    planet_pos[14+i].z, 3.0)) {
          glPushMatrix(); 
            glTranslated(planet_pos[14+i].x, planet_pos[14+i].y, planet_pos[14+i].z);
            DrawJupiterMesic(i,PointDistance(planet_pos[14+i],data->camera_pos),data->t);
          glPopMatrix();
        }  
    }
    if (frustum.SphereInFrustum(planet_pos[13].x, planet_pos[13].y,
                                planet_pos[13].z, 100.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[13].x, planet_pos[13].y, planet_pos[13].z);
            glRotatef(130.0, 0.0, 1.0,0.0);
            DrawJupiter (PointDistance(planet_pos[13], data->camera_pos), data->t );
        glPopMatrix();
    }

    if (frustum.SphereInFrustum(planet_pos[18].x, planet_pos[18].y,
                                planet_pos[18].z, 4.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[18].x, planet_pos[18].y, planet_pos[18].z);
            glRotatef(10.0, 0.0, 1.0,0.0);
            DrawMars (PointDistance(planet_pos[18], data->camera_pos), data->t );
        glPopMatrix();
    }
    if (frustum.SphereInFrustum(planet_pos[19].x, planet_pos[19].y,
                                planet_pos[19].z, 1.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[19].x, planet_pos[19].y, planet_pos[19].z);
            DrawPhobos (PointDistance(planet_pos[19], data->camera_pos), data->t );
        glPopMatrix();
    }
    if (frustum.SphereInFrustum(planet_pos[20].x, planet_pos[20].y,
                                planet_pos[20].z, 1.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[20].x, planet_pos[20].y, planet_pos[20].z);
            DrawDeimos (PointDistance(planet_pos[20], data->camera_pos), data->t );
        glPopMatrix();
    }

    if (frustum.SphereInFrustum(planet_pos[22].x, planet_pos[22].y,
                                planet_pos[22].z, 4.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[22].x, planet_pos[22].y, planet_pos[22].z);
            glRotatef(200.0,0.0,1.0,0.0);
            DrawMesic (PointDistance(planet_pos[22], data->camera_pos), data->t );
        glPopMatrix();
    }
    if (frustum.SphereInFrustum(planet_pos[21].x, planet_pos[21].y,
                                planet_pos[21].z, 10.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[21].x, planet_pos[21].y, planet_pos[21].z);
            glRotatef(180.0,0.0,1.0,0.0);
            glRotatef(30.0,1.0,0.0,0.0);
            DrawZeme (PointDistance(planet_pos[21], data->camera_pos), data->t );
        glPopMatrix();
    }

    if (frustum.SphereInFrustum(planet_pos[23].x, planet_pos[23].y,
                                planet_pos[23].z, 10.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[23].x, planet_pos[23].y, planet_pos[23].z);
            DrawVenuse (PointDistance(planet_pos[23], data->camera_pos), data->t );
        glPopMatrix();
    }
    if (frustum.SphereInFrustum(planet_pos[24].x, planet_pos[24].y,
                                planet_pos[24].z, 3.0)) {
        glPushMatrix(); 
            glTranslated(planet_pos[24].x, planet_pos[24].y, planet_pos[24].z);
            DrawMerkur (PointDistance(planet_pos[24], data->camera_pos), data->t );
        glPopMatrix();
    }

    return 0;
};

__export c_scene_planets scene_planets;


