/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include "scene_planets_data.hpp"

__export DVECTOR3D planet_pos[] = {
    // holt musi byt vsechny vzdalenosti desetinou skutecnosti
    { -295676.0, 1000.0, 512125.8 }, // pluto
    { -295686.0,  994.0, 512125.0 }, // charon
    
    { -317990.0,    0.0, 317990.0 }, // neptun
    { -318100.0,    0.0, 318200.0 }, // triton

    { -248635.0,    0.0, 143549.0 }, // uran
    { -248635.0,    0.0, 143600.0 }, // ariel
    { -248600.0,  -30.0, 143500.0 }, // umbriel
    { -248480.0,    0.0, 143500.0 }, // titania
    { -248500.0,    0.0, 143700.0 }, // oberon

    {   71347.0,    0.0, 123577.0 }, // saturn
    {   71200.0,   -2.0, 123400.0 }, // saturn - mimas
    {   71254.0,    4.0, 123500.0 }, // saturn - rhea
    {   71300.0,   40.0, 123800.0 }, // saturn - titan

    { - 77833.0,    0.0,      0.0 }, // jupiter 13
    { - 77533.0,    0.0,   -300.0 }, // jupiter - io
    { - 78233.0,  -30.0,    200.0 }, // jupiter - europa
    { - 77833.0,   40.0,   -300.0 }, // jupiter - ganymede
    { - 77800.0,    0.0,    200.0 }, // jupiter - callisto

    {   16117.0,    0.0,  16117.0 }, // mars 18
    {   16120.0,    0.0,  16110.0 }, // mars - phobos
    {   16110.0,    0.0,  16115.0 }, // mars - daemos

    {   10578.0,    0.0,  10578.0 }, // zem
    {   10800.0,    0.0,  10700.0 }, // mesic

    {   10655.0,    0.0,   1879.0 }, // venuse
    {    5015.0,    0.0,   2895.0 }, // merkur

};

struct t_data_cam_path {
    // camera position,  camera target, up vector
    double point;
    double type;

    DVECTOR3D cam_pos;
    DVECTOR3D cam_tg;
    DVECTOR3D cam_up;
  
} data_cam_path[] = {
    { 0.0, 0.0,
        { planet_pos[0].x+50.0, planet_pos[0].y-4.0, planet_pos[0].z-50.0 }, // pos
        { planet_pos[0].x-80.0, planet_pos[0].y+0.0, planet_pos[0].z-40.0 }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    { 8.0, 0.0,
        { planet_pos[0].x+10.0, planet_pos[0].y-1.0, planet_pos[0].z-10.0 }, // pos
        { planet_pos[0].x-40.0, planet_pos[0].y+0.0, planet_pos[0].z+40.0 }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    { 20.0, 0.0,
        { planet_pos[0].x-100.0, planet_pos[0].y+1.0, planet_pos[0].z+30.0 }, // pos
        { planet_pos[1].x+40.0, planet_pos[1].y, planet_pos[1].z }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    { 24.0, 0.0,
        { planet_pos[0].x-80.0, planet_pos[0].y+1.0, planet_pos[0].z+30.0 }, // pos
        { planet_pos[2].x+40.0, planet_pos[2].y, planet_pos[2].z }, // target
        { 0.0, 1.0, 2.0 }, // up
    },
    { 36.0, 0.0,
        { planet_pos[2].x+80.0, planet_pos[2].y-40.0, planet_pos[2].z-160.0 }, // pos
        { planet_pos[2].x+40.0, planet_pos[2].y, planet_pos[2].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    { 40.0, 0.0,
        { planet_pos[2].x+10.0, planet_pos[2].y+40.0, planet_pos[2].z-160.0 }, // pos
        { planet_pos[2].x+10.0, planet_pos[2].y, planet_pos[2].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    { 44.0, 1.0,
        { planet_pos[2].x+13.0, planet_pos[2].y+20.0, planet_pos[2].z-155.0 }, // pos
        { planet_pos[2].x, planet_pos[2].y, planet_pos[2].z }, // target
        { 0.0, 1.0,-0.2 }, // up
    },
    { 60.0, 0.0,
        { planet_pos[4].x+200.0, planet_pos[4].y, planet_pos[4].z-200.0 }, // pos
        { planet_pos[4].x+60.0, planet_pos[4].y, planet_pos[4].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    { 70.0, 0.0,
        { planet_pos[4].x-100.0, planet_pos[4].y-20.0, planet_pos[4].z-20.0 }, // pos
        { planet_pos[4].x+60.0, planet_pos[4].y-20.0, planet_pos[4].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    { 78.0, 0.0,
        { planet_pos[4].x-100.0, planet_pos[4].y-20.0, planet_pos[4].z-20.0 }, // pos
        { planet_pos[7].x, planet_pos[7].y, planet_pos[7].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {100.0, 0.0,
        { planet_pos[9].x-800.0, planet_pos[9].y+20.0, planet_pos[9].z+300.0 }, // pos
        { planet_pos[9].x, planet_pos[9].y, planet_pos[9].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {120.0, 0.0,
        { planet_pos[9].x+500.0, planet_pos[9].y-160.0, planet_pos[9].z-400.0 }, // pos
        { planet_pos[9].x, planet_pos[9].y, planet_pos[9].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {132.0, 0.0,
        { planet_pos[9].x+200.0, planet_pos[9].y+60.0, planet_pos[9].z-200.0 }, // pos
        { planet_pos[9].x, planet_pos[9].y, planet_pos[9].z }, // target
        { 0.0, 1.0, -1.0 }, // up
    },
    {140.0, 0.0,
        { planet_pos[9].x+200.0, planet_pos[9].y+60.0, planet_pos[9].z-200.0 }, // pos
        { planet_pos[13].x, planet_pos[13].y, planet_pos[13].z }, // target
        { 0.0, 1.0, -1.0 }, // up
    },
    {156.0, 0.0,
        { planet_pos[13].x+400.0, planet_pos[13].y, planet_pos[13].z-700.0 }, // pos
        { planet_pos[13].x+0.0, planet_pos[13].y, planet_pos[13].z+0.0 }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    {166.0, 1.0,
        { planet_pos[13].x+100.0, planet_pos[13].y+70.0, planet_pos[13].z-700.0 }, // pos
        { planet_pos[13].x+300.0, planet_pos[13].y, planet_pos[13].z+0.0 }, // target
        { 0.0, 1.0, 4.0 }, // up
    },
    {168.0, 0.0,
        { planet_pos[13].x+100.0, planet_pos[13].y+70.0, planet_pos[13].z-700.0 }, // pos
        { planet_pos[18].x+30.0, planet_pos[18].y, planet_pos[18].z+0.0 }, // target
        { 0.0, 1.0, 3.0 }, // up
    },
    {182.0, 0.0,
        { planet_pos[18].x+10.0, planet_pos[18].y+10.0, planet_pos[18].z-70.0 }, // pos
        { planet_pos[18].x+10.0, planet_pos[18].y, planet_pos[18].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {194.0, 0.0,
        { planet_pos[18].x-40.0, planet_pos[18].y, planet_pos[18].z+40.0 }, // pos
        { planet_pos[18].x, planet_pos[18].y, planet_pos[18].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {200.0, 0.0,
        { planet_pos[18].x+10.0, planet_pos[18].y, planet_pos[18].z+1000.0 }, // pos
        { planet_pos[21].x+10.0, planet_pos[21].y, planet_pos[21].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {212.0, 0.0,
        { planet_pos[22].x+50.0, planet_pos[22].y, planet_pos[22].z+50.0 }, // pos
        { planet_pos[22].x, planet_pos[22].y, planet_pos[22].z }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    {222.0, 0.0,
        { planet_pos[22].x+6.0, planet_pos[22].y, planet_pos[22].z-30.0 }, // pos
        { planet_pos[22].x+6.0, planet_pos[22].y, planet_pos[22].z }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    {240.0, 0.0,
        { planet_pos[21].x-15.0, planet_pos[21].y, planet_pos[21].z-30.0 }, // pos
        { planet_pos[21].x+10.0, planet_pos[21].y, planet_pos[21].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {250.0, 0.0,
        { planet_pos[21].x-40.0, planet_pos[21].y+1.0, planet_pos[21].z+0.0 }, // pos
        { planet_pos[21].x+10.0, planet_pos[21].y, planet_pos[21].z-20.0 }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {256.0, 0.0,
        { planet_pos[21].x-20.0, planet_pos[21].y+1.0, planet_pos[21].z+20.0 }, // pos
        { planet_pos[21].x+10.0, planet_pos[21].y, planet_pos[21].z-20.0 }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {262.0, 0.0,
        { planet_pos[21].x-10.0, planet_pos[21].y, planet_pos[21].z+12.0 }, // pos
        { planet_pos[23].x+10.0, planet_pos[23].y, planet_pos[23].z }, // target
        {-0.5, 1.0, 0.0 }, // up
    },
    {278.0, 0.0,
        { planet_pos[23].x-20.0, planet_pos[23].y, planet_pos[23].z-20.0 }, // pos
        { planet_pos[23].x+10.0, planet_pos[23].y, planet_pos[23].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {288.0, 0.0,
        { planet_pos[23].x+20.0, planet_pos[23].y, planet_pos[23].z-20.0 }, // pos
        { planet_pos[23].x-10.0, planet_pos[23].y, planet_pos[23].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {294.0, 0.0,
        { planet_pos[23].x-20.0, planet_pos[23].y, planet_pos[23].z-20.0 }, // pos
        { planet_pos[24].x+10.0, planet_pos[24].y, planet_pos[24].z }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {308.0, 0.0,
        { planet_pos[24].x-20.0, planet_pos[24].y, planet_pos[24].z-20.0 }, // pos
        { planet_pos[24].x+10.0, planet_pos[24].y, planet_pos[24].z }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    {314.0, 0.0,
        { planet_pos[24].x-20.0, planet_pos[24].y, planet_pos[24].z+30.0 }, // pos
        { planet_pos[24].x-10.0, planet_pos[24].y, planet_pos[24].z }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    {326.0, 0.0,
        { planet_pos[24].x-20.0, planet_pos[24].y, planet_pos[24].z+30.0 }, // pos
        { 0.0, 0.0, 0.0 }, // target
        { 0.0, 1.0, 1.0 }, // up
    },
    {350.0, 0.0,
        { 1400.0, 0.0, 1400.0 }, // pos
        { 0.0, 0.0, 0.0 }, // target
        { 0.0, 1.0, 0.0 }, // up
    },
    {358.0, 0.0,
        { 1000.0, 700.0, 1000.0 }, // pos
        { 0.0,    700.0, 0.0 }, // target
        { 0.0, 1.0,-1.0 }, // up
    },
    {380.0, 0.0,
        {-1000.0, 700.0,-1000.0 }, // pos
        {-2000.0, 700.0,-2000.0 }, // target
        { 0.0, 1.0,-1.0 }, // up
    },

};

__export int FillPaths( aPath *tp, aPath *ps, aPath *tg, aPath *up ) {
    tp->reset();
    ps->reset();
    tg->reset();
    up->reset();

    for (int i=0; i<sizeof(data_cam_path)/sizeof(t_data_cam_path); ++i) {
        tp->addp( data_cam_path[i].point, data_cam_path[i].type); // sinus
        ps->addp( data_cam_path[i].point,
                      data_cam_path[i].cam_pos.x,
                      data_cam_path[i].cam_pos.y,
                      data_cam_path[i].cam_pos.z );
        tg->addp( data_cam_path[i].point,
                      data_cam_path[i].cam_tg.x,
                      data_cam_path[i].cam_tg.y,
                      data_cam_path[i].cam_tg.z );
        up->addp( data_cam_path[i].point,
                      data_cam_path[i].cam_up.x,
                      data_cam_path[i].cam_up.y,
                      data_cam_path[i].cam_up.z );
    };

    return 0;
};


