/*
  Name: Space 2004: Solar System
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: This file is part of freeware project.
               Source code is included only for learning purposes.
               
               Consider it to be free to check how things are
               done but not for application in any project (both 
               commercial and non-commercial) without author's
               permission.
*/

#include "vypis.hpp"
#include "iface_def.hpp"
#include "../aLib/random/random.hpp"

HANDLE hTNahoru = NULL;
HANDLE hTDolu = NULL;
HANDLE hTTab = NULL;

FSOUND_SAMPLE *vypis_snd, *vypis_kl[10];

int InitVypis() {
    vypis_snd = FSOUND_Sample_Load( FSOUND_FREE, "data/snd9.ogg",
                                    FSOUND_NORMAL, 0, 0 );
    vypis_kl[0] = FSOUND_Sample_Load( FSOUND_FREE, "data/space.ogg",
                                      FSOUND_NORMAL, 0, 0 );
    vypis_kl[1] = FSOUND_Sample_Load( FSOUND_FREE, "data/keyb.ogg",
                                      FSOUND_NORMAL, 0, 0 );
    return 0;
}


bool thnahoruterm = false;
DWORD WINAPI ThreadNahoru(LPVOID string) {
    const char *sz = (const char *)string;
    int delka = strlen(sz)+9;

    int ch = -1;

    bool flip = false;

    if (iface_def.black < 0.1)
    for (int i=0; i<2*delka; ++i) {
        flip = !flip;
    
        char aaa[delka+1];
        ZeroMemory(aaa,delka+1);
        
        int chn = (i/2)<?strlen(sz);
        if (ch != chn && sz[chn] != ' ' && sz[chn] != '\0') {
            ch = chn;
            int r = FSOUND_PlaySound(15, vypis_snd);
        }

        strncpy( aaa, sz, chn );
        if (flip) aaa[strlen((char*)aaa)] = '_';

        strcpy( iface_def.szNahore, (char*)aaa );
        
        if (thnahoruterm) break;
        Sleep(50);
    }

    GlobalFree(string);
    return 0;
};

bool thdoluterm = false;
DWORD WINAPI ThreadDolu(LPVOID string) {
    randomize();
    
    const char *sz = (const char *)string;
    int delka = strlen(sz)+8;
    if (delka % 2) ++delka;
    
    int ch = -1;

    bool flip = false;

    if (iface_def.black < 0.1)
    for (int i=0; i<2*delka; ++i) {
        flip = !flip;
    
        char aaa[delka+1];
        ZeroMemory(aaa,delka+1);
        
        int chn = (i/2)<?strlen(sz);
        if (ch != chn) {
            ch = chn;
            if (sz[chn] != '\0')
                if (sz[chn] != ' ') {
                    int c = FSOUND_PlaySound(14, vypis_kl[1]);
                    float mod = 0.9+0.0002*random(1000);
                    FSOUND_SetFrequency(c, (int)(FSOUND_GetFrequency(c) * mod) );
                } else {
//                    FSOUND_PlaySound(14, vypis_kl[0]);
                }
        }

        strncpy( aaa, sz, chn );
        if (flip) aaa[strlen((char*)aaa)] = '_';

        strcpy( iface_def.szDole, (char*)aaa );
        
        if (thdoluterm) break;
        Sleep(100);
    }

    GlobalFree(string);
    return 0;
};

bool thtabterm = false;
DWORD WINAPI ThreadTabulka(LPVOID string) {
    const char *sz = (const char *)string;
    int delka = strlen(sz);
    int ch;
    
    int begin = 0;
    int row = 0;
    bool flip = false;
    
    ZeroMemory( iface_def.szTab, 32*256 );

    if (iface_def.black < 0.1)
    for (int i=0; i<delka; ++i) {
        flip = !flip;
        ch = i<?strlen(sz);
        
        if (sz[ch] == '\n') {
            ++ch;
            begin = ch;
            ++row;
            if (row >= 32) break;
        }
    
        char aaa[delka+1];
        ZeroMemory(aaa,delka+1);
        
        strncpy( aaa, &sz[begin], ch-begin+1 );
        strncpy( iface_def.szTab[row], aaa, 248 );
        
        if (thtabterm) break;
        Sleep(10);
    }

    GlobalFree(string);
    return 0;
};

__export int Nahoru( const char* sz ) {
    if (!sz) return 0;

    char *aaa = (char*)GlobalAlloc(GPTR, strlen(sz)+1);
    if (!aaa) return 0;

    strcpy(aaa,sz);    
    
    if (hTNahoru) {
        thnahoruterm = true;
        WaitForSingleObject(hTNahoru, INFINITE);
        thnahoruterm = false;
    }

    DWORD id = 0;
    hTNahoru = CreateThread( NULL, 0, ThreadNahoru, aaa, 0, &id);
    
    return 1;
};

__export int Dolu( const char* sz ) {
    if (!sz) return 0;

    char *aaa = (char*)GlobalAlloc(GPTR, strlen(sz)+1);
    if (!aaa) return 0;
    
    strcpy(aaa,sz);    
    
    if (hTDolu) {
        thdoluterm = true;
        WaitForSingleObject(hTDolu, INFINITE);
        thdoluterm = false;
    }

    DWORD id = 0;
    hTDolu = CreateThread( NULL, 0, ThreadDolu, aaa, 0, &id);
    
    return 1;
};

__export int Tabulka( const char* sz ) {
    if (!sz) return 0;

    char *aaa = (char*)GlobalAlloc(GPTR, strlen(sz)+1);
    if (!aaa) return 0;
    
    strcpy(aaa,sz);    
    
    if (hTTab) {
        thtabterm = true;
        WaitForSingleObject(hTTab, INFINITE);
        thtabterm = false;
    }

    DWORD id = 0;
    hTTab = CreateThread( NULL, 0, ThreadTabulka, aaa, 0, &id);
    
    return 1;
};


