/*
  Name: aBitmap.cpp
  Copyright: Copyright(C) 2003, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aBitmap graphics format library
  
  This is not official version of the code.
*/

#include "aBitmap.hpp"

aBitmap::aBitmap() {
    strcpy(magic,"aBitmap\0");
    sizex = 0;
    sizey = 0;
    p1 = 0;
    p2 = 0;
    p3 = 0;
    rgba = NULL;
}
aBitmap::aBitmap(long x,long y) {
    strcpy(magic,"aBitmap\0");
    sizex = x;
    sizey = y;
    p1 = 0;
    p2 = 0;
    p3 = 0;
    rgba = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
}
aBitmap::~aBitmap() {
    GlobalFree(rgba);
}

int aBitmap::getwidth() { return sizex; }
int aBitmap::getheight() { return sizey; }

int aBitmap::load(char *name){
    FILE *f;
    f = fopen(name,"rb");
    if (!f) return AGL_ERROR_FILEERR;
    
    // testnout magic
    char temp[8];
    fread(temp,8,1,f);
    if (strcmp(magic,temp)){ fclose(f); return AGL_ERROR_BADFILE; }
    
    // velikost
    long tx,ty;
    fread(&tx,sizeof(long),1,f);
    fread(&ty,sizeof(long),1,f);
    if (tx!=sizex||ty!=sizey) { // realokovat, jednoduse
        if (rgba) GlobalFree(rgba);
        sizex = tx;
        sizey = ty;
        rgba = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
        if (!rgba) return AGL_ERROR_MEMORY; // not enough memory
    }

    fread(&p1,sizeof(long),1,f);
    if (p1 > 1){ fclose(f); return AGL_ERROR_COMPRESSION; } // nonsupported compression
    
    fread(&p2,sizeof(long),1,f);
    fread(&p3,sizeof(long),1,f);
    fread(&p4,sizeof(long),1,f);

    // decompress
    if (p1 == 0) fread(rgba,sizeof(__rgba),sizex*sizey,f);
    if (p1 == 1) {
        unsigned long insize = filesize(f) - 32;
        char *in = (char*)GlobalAlloc(GPTR, insize );
        if (!in) return AGL_ERROR_MEMORY; // not enough memory
        
        fread(in,insize,1,f);
        
        unsigned long destLen = sizex*sizey*sizeof(__rgba);
        int ret = uncompress( (Bytef*)rgba, &destLen, (Bytef*)in, insize );
        
        if (ret == Z_MEM_ERROR){ GlobalFree(in); return AGL_ERROR_MEMORY; }
        if (ret == Z_BUF_ERROR){ GlobalFree(in); return AGL_ERROR_BUFFER; }
        if (ret == Z_DATA_ERROR){ GlobalFree(in); return AGL_ERROR_ZDATA; }
        
        if (destLen != sizex*sizey*sizeof(__rgba)){
            GlobalFree(in); return AGL_ERROR_FILECUT; 
        }
        
        GlobalFree(in);
    }
    
    fclose(f);
    
    return 0;
} // load
        
int aBitmap::save(char *name){
    FILE *f;
    f = fopen(name,"wb");
    if (!f) return AGL_ERROR_FILEERR;
    
    p1 = 0;
    fwrite(this,sizeof(aBitmap)-sizeof(rgba),1,f); // hlavicku
    fwrite(rgba,4*sizex*sizey,1,f); // hlavicku
    
    fclose(f);
    return 0;
} // save

int aBitmap::save_c(char *name){
    FILE *f;
    f = fopen(name,"wb");
    if (!f) return AGL_ERROR_FILEERR;
    
    p1 = 1; // compressed !
    fwrite(this,sizeof(aBitmap)-sizeof(rgba),1,f); // hlavicku

    unsigned long outLen = (int)(1.001*(float)(sizeof(__rgba)*sizex*sizey))+32;
    char *out = (char*)GlobalAlloc(GPTR,outLen);
    if (!out) return AGL_ERROR_MEMORY;
    
    int ret = compress2( (Bytef*)out, &outLen,
                         (Bytef*)rgba, sizeof(__rgba)*sizex*sizey, 9 );
    
    if (ret == Z_MEM_ERROR){ GlobalFree(out); return AGL_ERROR_MEMORY; }
    if (ret == Z_BUF_ERROR){ GlobalFree(out); return AGL_ERROR_BUFFER; }
    if (ret == Z_STREAM_ERROR){ GlobalFree(out); return AGL_ERROR_ZSTREAM; }
   
    fwrite(out,outLen,1,f); // hlavicku
    GlobalFree(out);
    
    fclose(f);
    return 0;
} // save

        
int aBitmap::draw(HDC hDC,long x,long y,float i=1.0) {
    // x,y - pos; i-intensity (0.0 - invisible to 1.0 - full)
    
    // je-li i zaporne nebere se v uvahu obsah okna
    // ale jen std. pozadi = rychlejsi
    
    if (i > 1.0) i= 1.0;
    if (i <-1.0) i=-1.0;
    COLORREF b; 
    COLORREF bgcol = GetSysColor(COLOR_3DFACE);
    int _r,_g,_b,_a;
    
    if (sizex&&sizey&&i!=0.0)
    for (int py=0;py<sizey;py++)
        for (int px=0;px<sizex;px++) {
        
        if (i>0.0)    
            b = GetPixel(hDC,x+px,y+py);
        else
            b = bgcol;
          
        _a = (int)(fabs(i)*(float)rgba[py*sizex+px].a);  
        _r = ((rgba[py*sizex+px].r*_a)>>8)
             +((GetRValue(b)*(255-_a))>>8);
        _g = ((rgba[py*sizex+px].g*_a)>>8)
             +((GetGValue(b)*(255-_a))>>8);
        _b = ((rgba[py*sizex+px].b*_a)>>8)
             +((GetBValue(b)*(255-_a))>>8);

        SetPixel(hDC,x+px,y+py,RGB(_r,_g,_b));
    }
    return 0;
}
int aBitmap::blit(HDC hDC,long x,long y) {
    BITMAPINFO bi = {
        { sizeof(BITMAPINFOHEADER),
          sizex,
          sizey,
          1, 32,
          BI_RGB
        }
    };
    COLORREF *bitmap = (COLORREF*)GlobalAlloc(GPTR,sizeof(COLORREF)*sizex*sizey);

    if (sizex&&sizey)
    for (int py=0;py<sizey;py++)
        for (int px=0;px<sizex;px++) {
        
        bitmap[(sizey-py-1)*sizex+px] =
            RGB(rgba[py*sizex+px].r,rgba[py*sizex+px].g,rgba[py*sizex+px].b);
    }

    SetDIBitsToDevice(hDC,x,y,sizex,sizey,0,0,0,sizey,
                      bitmap,&bi,DIB_RGB_COLORS);
    GlobalFree(bitmap);

    return 0;
}

HBITMAP aBitmap::gethbitmap(HDC hDC) {
    __rgba* nrgba = (__rgba*)GlobalAlloc(GPTR, sizeof(__rgba)*sizex*sizey);
    
    int pos;
    for (int y = 0; y < sizey; ++y) {
        for (int x = 0; x < sizex; ++x) {
            pos = y*sizex + x;
            
            nrgba[pos].r = rgba[pos].b;
            nrgba[pos].g = rgba[pos].g;
            nrgba[pos].b = rgba[pos].r;
            nrgba[pos].a = rgba[pos].a;
        }
    }

    HBITMAP hbitmap = CreateBitmap(sizex, sizey, 1, 32, nrgba);

    GlobalFree(nrgba);
    return hbitmap;
};
//*/
HBITMAP aBitmap::getULWbitmap(HDC hDC) {
    __rgba* nrgba = (__rgba*)GlobalAlloc(GPTR, sizeof(__rgba)*sizex*sizey);
    
    int pos;
    for (int y = 0; y < sizey; ++y) {
        for (int x = 0; x < sizex; ++x) {
            pos = y*sizex + x;
            
            nrgba[pos].r = rgba[pos].b * rgba[pos].a / 0xFF;
            nrgba[pos].g = rgba[pos].g * rgba[pos].a / 0xFF;
            nrgba[pos].b = rgba[pos].r * rgba[pos].a / 0xFF;
            nrgba[pos].a = rgba[pos].a;
        }
    }

    HBITMAP hbitmap = CreateBitmap(sizex, sizey, 1, 32, nrgba);

    GlobalFree(nrgba);
    return hbitmap;
};

int aBitmap::setpixel(int x,int y,int r,int g,int b,int a) {
    if (x<0) return -1;
    if (y<0) return -1;
    if (x>sizex-1) return -1;
    if (y>sizey-1) return -1;
    
    rgba[y*sizex+x].r = r;
    rgba[y*sizex+x].g = g;
    rgba[y*sizex+x].b = b;
    rgba[y*sizex+x].a = a;
}

COLORREF aBitmap::getpixel(int x,int y) {
    if (x<0) return 0;
    if (y<0) return 0;
    if (x>sizex-1) return 0;
    if (y>sizey-1) return 0;

    COLORREF e = getpixelrgb(x,y);
    e |= (((DWORD) (BYTE) (rgba[y*sizex+x].a)) << 24);
    
    return e;
}

COLORREF aBitmap::getpixelrgb(int x,int y) {
    if (x<0) return 0;
    if (y<0) return 0;
    if (x>sizex-1) return 0;
    if (y>sizey-1) return 0;

    COLORREF __e = RGB(rgba[y*sizex+x].r,rgba[y*sizex+x].g,rgba[y*sizex+x].b);
    
    return __e;
}

int aBitmap::getpixelalpha(int x,int y) {
    if (x<0) return -1;
    if (y<0) return -1;
    if (x>sizex-1) return -1;
    if (y>sizey-1) return -1;
    
    return rgba[y*sizex+x].a;
}

int aBitmap::storedata(COLORREF *c,int n,int xlen){
    if (!c || !xlen || !n) return -1;
    
    int ylen = n/xlen;
    if (n%xlen) ylen++;
    
    if (xlen!=sizex||ylen!=sizey) { // realokovat, jednoduse
        if (rgba) GlobalFree(rgba);
        sizex = xlen;
        sizey = ylen;
        rgba = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
        if (!rgba) return AGL_ERROR_MEMORY; // not enough memory
    }
    
    CopyMemory(rgba,c,4*xlen*ylen);
        
    return 0;
}

int aBitmap::loadbmp(HDC hDC, char *name){
    HBITMAP hbm;
    DWORD dwRead;
    BITMAPFILEHEADER bmfh;
    BITMAPINFOHEADER bmih;
    char *hmem1,*hmem2;
    LPBITMAPINFO lpbmi;
    CHAR *lpvBits;
    
    HANDLE hfbm = CreateFile(name, GENERIC_READ, 
                  FILE_SHARE_READ, (LPSECURITY_ATTRIBUTES) NULL, 
                  OPEN_EXISTING, FILE_ATTRIBUTE_READONLY, (HANDLE) NULL); 

    ReadFile(hfbm, &bmfh, sizeof(BITMAPFILEHEADER), &dwRead, (LPOVERLAPPED)NULL); 
    ReadFile(hfbm, &bmih, sizeof(BITMAPINFOHEADER), &dwRead, (LPOVERLAPPED)NULL); 

    lpbmi = (LPBITMAPINFO)GlobalAlloc(GPTR, 
            sizeof(BITMAPINFOHEADER) + 
            ((bmih.biCompression != BI_RGB)?((1<<bmih.biBitCount)*sizeof(RGBQUAD)):0)); 
 
    lpbmi->bmiHeader.biSize = bmih.biSize; 
    lpbmi->bmiHeader.biWidth = bmih.biWidth; 
    lpbmi->bmiHeader.biHeight = bmih.biHeight; 
    lpbmi->bmiHeader.biPlanes = bmih.biPlanes; 
    
    lpbmi->bmiHeader.biBitCount = bmih.biBitCount; 
    lpbmi->bmiHeader.biCompression = bmih.biCompression; 
    lpbmi->bmiHeader.biSizeImage = bmih.biSizeImage; 
    lpbmi->bmiHeader.biXPelsPerMeter = bmih.biXPelsPerMeter; 
    lpbmi->bmiHeader.biYPelsPerMeter = bmih.biYPelsPerMeter; 
    lpbmi->bmiHeader.biClrUsed = bmih.biClrUsed; 
    lpbmi->bmiHeader.biClrImportant = bmih.biClrImportant; 
 
    if (bmih.biCompression != BI_RGB)
    ReadFile(hfbm, lpbmi->bmiColors, 
             ((1<<bmih.biBitCount) * sizeof(RGBQUAD)) , 
             &dwRead, (LPOVERLAPPED) NULL); 
 
    lpvBits = (CHAR*)GlobalAlloc(GPTR, (bmfh.bfSize - bmfh.bfOffBits)); 
 
    ReadFile(hfbm, lpvBits, 
             (bmfh.bfSize - bmfh.bfOffBits),  
             &dwRead, (LPOVERLAPPED) NULL);  
 
    hbm = CreateDIBitmap(hDC, &bmih, CBM_INIT, lpvBits, lpbmi, DIB_RGB_COLORS); 

    GlobalFree(lpbmi); 
    GlobalFree(lpvBits); 
    CloseHandle(hfbm); 
    
    loadhbitmap(hDC,hbm);
    DeleteObject(hbm);
    return 0;
}

int aBitmap::loadhbitmap(HDC hDC,HBITMAP hBitmap){
    BITMAP bm;
    GetObject( hBitmap, sizeof(BITMAP), &bm);

    if (bm.bmWidth!=sizex||bm.bmHeight!=sizey) { // realokovat, jednoduse
        if (rgba) GlobalFree(rgba);
        sizex = bm.bmWidth;
        sizey = bm.bmHeight;
        rgba = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
    }

    BITMAPINFOHEADER bh = {sizeof(BITMAPINFOHEADER),bm.bmWidth,bm.bmHeight,1,32,BI_RGB};
    BITMAPINFO bi = { bh, 0 };
     
    HDC hDCa = CreateCompatibleDC(hDC);
    GetDIBits(hDCa, hBitmap, 0, bm.bmHeight, rgba, &bi, DIB_RGB_COLORS);
    DeleteDC(hDCa);
    
    int tmp;
    int i;
    for (i=0;i<bm.bmWidth*bm.bmHeight;i++){
         tmp = rgba[i].r;
         rgba[i].r = rgba[i].b;
         rgba[i].b = tmp;
         rgba[i].a = 255; // plne
    }
    // prohodit radky
    __rgba radek[sizex];
    ZeroMemory(radek,sizeof(__rgba)*sizex);
    for (i=0;i<sizey/2;i++) {
        MoveMemory(radek,&rgba[i*sizex],4*sizex);
        MoveMemory(&rgba[i*sizex],&rgba[(sizey-i-1)*sizex],4*sizex);
        MoveMemory(&rgba[(sizey-i-1)*sizex],radek,4*sizex);
    }
    
    return 0;
}

int aBitmap::loadresource(HDC hDC, LPCTSTR name) {
    HRSRC hRes = FindResource(NULL, name, "aBitmap" );
    if (hRes == NULL) return AGL_ERROR_BADPARAM;
    
    HGLOBAL hGlobal = LoadResource(NULL, hRes);
    if (hRes == NULL) return AGL_ERROR_BADPARAM;
    
    char *data = (char*) LockResource(hGlobal);
    if (!data) return AGL_ERROR_BADPARAM;
    
    int len = SizeofResource(NULL, hRes);
    if (len < 32) return AGL_ERROR_BUFFER;
    
    // final load
    char temp[8];
    CopyMemory(temp,data,8);
    if (strcmp(magic,temp)){ return AGL_ERROR_BADFILE; }
    
    long tx,ty;
    tx = *(long*)(&data[8]);
    ty = *(long*)(&data[12]);

    if (tx!=sizex||ty!=sizey) { // realokovat, jednoduse
        if (rgba) GlobalFree(rgba);
        sizex = tx;
        sizey = ty;
        rgba = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
        if (!rgba) return AGL_ERROR_MEMORY; // not enough memory
    }

    p1 = *(long*)(&data[16]);
    if (p1 > 1){ return AGL_ERROR_COMPRESSION; } // nonsupported compression
    
    p2 = *(long*)(&data[20]);
    p3 = *(long*)(&data[24]);
    p4 = *(long*)(&data[28]);
    
    if (p1 == 0) CopyMemory(rgba,&data[32],sizeof(__rgba)*sizex*sizey);
    if (p1 == 1) {
        unsigned long insize = len - 32;
        char *in = (char*)GlobalAlloc(GPTR, insize );
        if (!in) return AGL_ERROR_MEMORY; // not enough memory
        
        CopyMemory(in,&data[32],insize);
        
        unsigned long destLen = sizex*sizey*sizeof(__rgba);
        int ret = uncompress( (Bytef*)rgba, &destLen, (Bytef*)in, insize );
        
        if (ret == Z_MEM_ERROR){ GlobalFree(in); return AGL_ERROR_MEMORY; }
        if (ret == Z_BUF_ERROR){ GlobalFree(in); return AGL_ERROR_BUFFER; }
        if (ret == Z_DATA_ERROR){ GlobalFree(in); return AGL_ERROR_ZDATA; }
        
        if (destLen != sizex*sizey*sizeof(__rgba)){
            GlobalFree(in); return AGL_ERROR_FILECUT; 
        }
        
        GlobalFree(in);
    }
    
    UnlockResource(data);
    
    return 0;
};

int aBitmap::blur(int level) {
    if (level <= 0) return AGL_ERROR_BADPARAM;

    __rgba *nove = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
    if (!nove) return AGL_ERROR_MEMORY; // not enough memory
    
    int r = 0;
    int g = 0;
    int b = 0;
    
    int a = 0;
    int lev = 0;
    
    int cnt = 0;
                                        
    for (int y = 0; y < sizey; y++) {
        for (int x = 0; x < sizex; x++) {
            cnt = 0;
            r = g = b = a = 0;
            
            // blur all pixels to this one            
            for (int ly = -level; ly <= level; ly++) {
//                lev = (int)(level*(sin( PI*((float)ly/(float)level))));
                for (int lx = -level + lev; lx <= level - lev; lx++) {

                    if (x+lx<0) continue;
                    if (y+ly<0) continue;
                    if (x+lx>sizex-1) continue;
                    if (y+ly>sizey-1) continue;
                
                    r += rgba[(y+ly)*sizex+(x+lx)].r;
                    g += rgba[(y+ly)*sizex+(x+lx)].g;
                    b += rgba[(y+ly)*sizex+(x+lx)].b;
                    a += rgba[(y+ly)*sizex+(x+lx)].a;
                    
                    cnt++;
                }
            }
            
            r /= cnt;
            g /= cnt;
            b /= cnt;
            a /= cnt;
            
            nove[y*sizex+x].r = r;
            nove[y*sizex+x].g = g;
            nove[y*sizex+x].b = b;
            nove[y*sizex+x].a = a;
        }
    }
    
    GlobalFree(rgba);
    rgba = nove;
    
    return 0;
}

int aBitmap::dissblur(int level,int line) {
    if (level <= 0) return AGL_ERROR_BADPARAM;

    __rgba *nove = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
    if (!nove) return AGL_ERROR_MEMORY; // not enough memory
    
    for (int y = 0; y < sizey; y++) {
        if (!(y % line))
        for (int x = 0; x < sizex; x++) {
            // blur all pixels to this one            
            int r = 0;
            int g = 0;
            int b = 0;
            
            int a = 0;
            
            for (int lx = -level; lx <= level; lx++) {
                COLORREF color = getpixel(x+lx,y);
                r += GetRValue(color);
                g += GetGValue(color);
                b += GetBValue(color);
                a += (BYTE)(color >> 24);
            }
            
            r /= (2*level+1);
            g /= (2*level+1);
            b /= (2*level+1);
            a /= (2*level+1);
            
            nove[y*sizex+x].r = r;
            nove[y*sizex+x].g = g;
            nove[y*sizex+x].b = b;
            nove[y*sizex+x].a = a;
        }
/*        else
        for (int x = 0; x < sizex; x++) {
            // blur all pixels to this one            
            COLORREF color = getpixel(x,y);

            nove[y*sizex+x].r = GetRValue(color);
            nove[y*sizex+x].g = GetGValue(color);
            nove[y*sizex+x].b = GetBValue(color);
            nove[y*sizex+x].a = (BYTE)(color >> 24);
        }//*/
    }
    
    GlobalFree(rgba);
    rgba = nove;
    
    return 0;
}


int aBitmap::glow(float level) {
    if (level <= 0.0) return AGL_ERROR_BADPARAM;

    __rgba *nove = (__rgba*) GlobalAlloc(GPTR,4*sizex*sizey);
    if (!nove) return AGL_ERROR_MEMORY; // not enough memory
    
    int r,g,b,a;
    for (int y = 0; y < sizey; y++) {
        for (int x = 0; x < sizex; x++) {
            r = rgba[y*sizex+x].g;
            g = rgba[y*sizex+x].g;
            b = rgba[y*sizex+x].b;
            a = rgba[y*sizex+x].a;
            
            r = (int)((float)r*level);
            g = (int)((float)g*level);
            b = (int)((float)b*level);
            a = (int)((float)a*level);
            
            if (r > 255) r = 255;
            if (g > 255) g = 255;
            if (b > 255) b = 255;
            if (a > 255) a = 255;
            
            nove[y*sizex+x].r = r;
            nove[y*sizex+x].g = g;
            nove[y*sizex+x].b = b;
            nove[y*sizex+x].a = a;
        }
    }
    
    GlobalFree(rgba);
    rgba = nove;
    
    return 0;
}


int aBitmap::frame(int left, int top, int right, int bottom, COLORREF fill) {
    if (left < 0) return AGL_ERROR_BADPARAM;
    if (top < 0) return AGL_ERROR_BADPARAM;
    if (right < 0) return AGL_ERROR_BADPARAM;
    if (bottom < 0) return AGL_ERROR_BADPARAM;

    __rgba *nove = (__rgba*) GlobalAlloc(GPTR,4*(sizex+left+right)*(sizey+top+bottom));
    if (!nove) return AGL_ERROR_MEMORY; // not enough memory

    for (int y = 0; y < (sizey+top+bottom); y++) {
        for (int x = 0; x < (sizex+left+right); x++) {
            if (y < top || y >= sizey+top || x < left || x >= sizex+left) {
                nove[y*(sizex+left+right)+x].r = GetRValue(fill);
                nove[y*(sizex+left+right)+x].g = GetGValue(fill);
                nove[y*(sizex+left+right)+x].b = GetBValue(fill);
                nove[y*(sizex+left+right)+x].a = (BYTE)(fill >> 24);
            } else {
                nove[y*(sizex+left+right)+x].r = rgba[(y-top)*sizex+(x-left)].r;
                nove[y*(sizex+left+right)+x].g = rgba[(y-top)*sizex+(x-left)].g;
                nove[y*(sizex+left+right)+x].b = rgba[(y-top)*sizex+(x-left)].b;
                nove[y*(sizex+left+right)+x].a = rgba[(y-top)*sizex+(x-left)].a;
            }
        }
    }

    GlobalFree(rgba);
    rgba = nove;
    sizex = (sizex+left+right);
    sizey = (sizey+top+bottom);

    return 0;
}

int aBitmap::cut(int left, int top, int right, int bottom) {
    if (left < 0) return AGL_ERROR_BADPARAM;
    if (top < 0) return AGL_ERROR_BADPARAM;
    if (right < 0) return AGL_ERROR_BADPARAM;
    if (bottom < 0) return AGL_ERROR_BADPARAM;

    __rgba *nove = (__rgba*) GlobalAlloc(GPTR,4*(sizex-left-right)*(sizey-top-bottom));
    if (!nove) return AGL_ERROR_MEMORY; // not enough memory
    
    for (int y = 0; y < (sizey-top-bottom); y++) {
        for (int x = 0; x < (sizex-left-right); x++) {
            nove[y*(sizex-left-right)+x].r = rgba[(y+top)*sizex+(x+left)].r;
            nove[y*(sizex-left-right)+x].g = rgba[(y+top)*sizex+(x+left)].g;
            nove[y*(sizex-left-right)+x].b = rgba[(y+top)*sizex+(x+left)].b;
            nove[y*(sizex-left-right)+x].a = rgba[(y+top)*sizex+(x+left)].a;
        }
    }

    GlobalFree(rgba);
    rgba = nove;
    sizex = (sizex-left-right);
    sizey = (sizey-top-bottom);

    return 0;
}


int aBitmap::resize(HDC hDC,int nx,int ny) {
    // does not stretches alpha channel

    if (nx < 1) return AGL_ERROR_BADPARAM;
    if (ny < 1) return AGL_ERROR_BADPARAM;
    
    if (nx == sizex && ny == sizey) return 0;
    
    __rgba *nove = (__rgba*)GlobalAlloc(GPTR,sizeof(__rgba)*nx*ny);
    if (!nove) return AGL_ERROR_MEMORY;

    HDC hDCin = CreateCompatibleDC(hDC);
    HDC hDCout = CreateCompatibleDC(hDC);

    BITMAP hbIn = { 0, sizex, sizey, sizex*4, 1, 32, (LPVOID)rgba };
    BITMAP hbOut = { 0, nx, ny, nx*4, 1, 32, (LPVOID)nove };

    HBITMAP hBin = CreateBitmapIndirect(&hbIn);
    HBITMAP hBout= CreateBitmapIndirect(&hbOut);
    
    HBITMAP hBinOld = (HBITMAP)SelectObject(hDCin, hBin);
    HBITMAP hBoutOld = (HBITMAP)SelectObject(hDCout, hBout);
    
    SetStretchBltMode(hDCin, HALFTONE);
    SetStretchBltMode(hDCout, HALFTONE);
    StretchBlt(hDCout, 0,0, nx,ny, hDCin, 0,0, sizex, sizey, SRCCOPY);
        
    SelectObject(hDCin, hBinOld);
    SelectObject(hDCout, hBoutOld);
    
    BITMAPINFO bi = {{ sizeof(BITMAPINFOHEADER), nx, ny, 1, 32, BI_RGB, 0, 0, 0, 0, 0 }};
    GetDIBits(hDCout, hBout, 0, ny, (LPVOID)nove, &bi, DIB_RGB_COLORS);
    
    DeleteObject(hBin);
    DeleteObject(hBout);

    DeleteDC(hDCout);
    DeleteDC(hDCin);
    
    GlobalFree(rgba);
    rgba = nove;
    
    sizex = nx;
    sizey = ny;

    __rgba *line = (__rgba*) GlobalAlloc(GPTR,sizeof(__rgba)*nx);
    if (line) {
        for (int i=0; i<sizey/2; i++) {
            CopyMemory(line, &rgba[i*sizex], sizeof(__rgba)*nx);
            CopyMemory(&rgba[i*sizex], &rgba[(sizey-i-1)*sizex], sizeof(__rgba)*nx);
            CopyMemory(&rgba[(sizey-i-1)*sizex], line, sizeof(__rgba)*nx);
        }
        GlobalFree(line);
    }
    
    for (int y = 0; y < sizey; ++y)
        for (int x = 0; x < sizex; ++x)
            rgba[y*sizex + x].a = 0xFF;
    
    return 0;
}



