/*
  Name: aBitmap.hpp
  Copyright: Copyright(C) 2003, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aBitmap graphics format class

  Requirements: Windows Environment, OpenGL, Zlib compression library
  Handling: aBitmap format
            RGBA 8-bit pixel storage
            Compression of graphics data
  
  This is not official version of the code.
*/

#ifndef __ABITMAP_HPP__
#define __ABITMAP_HPP__

#include <windows.h>
#include <stdio.h>

#include <zlib.h>

#include "aDef.hpp"
#include "a3DMath.hpp"

class aBitmap {
    protected:
        char magic[8];
        long sizex;
        long sizey;
        
        long p1; // compression
        long p2; // reserved
        long p3; // reserved
        long p4; // reserved, do not extend, aBitmap.cpp assumes header to be 32 bytes
        
        struct __rgba {
            unsigned char r;
            unsigned char g;
            unsigned char b;
            unsigned char a;
        } *rgba;
    public:
        aBitmap();
        aBitmap(long x,long y);
        ~aBitmap();
        int getwidth();
        int getheight();
        
        int load(char *name); 
        int save(char *name);
        int save_c(char *name); // saves compressed

        int loadbmp(HDC,char *name);
        int loadhbitmap(HDC,HBITMAP);
        int loadresource(HDC,LPCTSTR);
        
        int storedata(COLORREF*,int,int);
        
        int draw(HDC,long,long,float);
        int blit(HDC,long,long);
        HBITMAP gethbitmap(HDC);
        HBITMAP getULWbitmap(HDC);
        
        int setpixel(int x,int y,int r,int g,int b,int a);
        COLORREF getpixel(int x,int y);
        COLORREF getpixelrgb(int x,int y);
        int getpixelalpha(int x,int y);
        
        int frame(int left,int top,int right,int bottom,COLORREF fill);
        int cut(int left,int top,int right,int bottom);
        int blur(int level);
        int dissblur(int level,int line);
        int glow(float level);
        
        int resize(HDC,int,int);
};


#endif

