/*
  Name: aGL.hpp
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aGL graphics library core

  Requirements: OpenGL, Windows environment
  Handling: Context-based OpenGL drawing
            2D/3D graphics functions, Materials, Lightning
            Objects: 2D and 3D Fonts, Icons, Textures
            Point, Line, Rect common functions
            Able to use direct OpenGL code
  
  Comment: Some comments are in czech language so don't get
           confused and try to understand the code :-) 
  
  This is not official version of the code.
*/

#ifndef __AGL_HPP__
#define __AGL_HPP__

#include <windows.h>
#include <stdio.h>
#include <ansidecl.h>
#include <gl\gl.h>
#include <gl\glu.h>

#include "aDef.hpp"
// To disable usage of some part, comment following lines
#include "aBitmapGL.hpp"
#include "aModel.hpp"
#include "a3Dmath.hpp"
#include "aFrustum.hpp"


typedef struct aGLContext {
    public:
        HDC hDC;
        HGLRC hRC;
        
        unsigned int sizex,sizey;
        CRITICAL_SECTION cs;
        
        float nearplane,farplane;
        float persp;
        
        aGLContext(); // simple init constructor
};
typedef struct aGLObject {
    unsigned int displaylist;
    unsigned int count;
    unsigned int sizex,sizey;
    unsigned int param1;
};

typedef aGLContext *aHANDLE;
typedef aGLObject aGLObject,*aObjectHANDLE,
                  aGLFont, *aFontHANDLE
//                  aGLIcon, *aIconHANDLE
#ifdef __ABITMAP_HPP__
                  ,aGLTexture, *aTextureHANDLE
#endif
                  ;
                  
extern aHANDLE aGLCreateContext (HWND hWnd, bool stencil=false, bool accum=false);
extern int aGLDeleteContext (HWND hWnd, aHANDLE);
extern int aGLResize(aHANDLE,int,int);
extern int aGLBeginPaint(aHANDLE);
extern int aGLEndPaint(HWND,aHANDLE);
extern int aGLBeginPureOpenGL(aHANDLE);
extern int aGLEndPureOpenGL(aHANDLE);
extern int aGLClearBuffers(aHANDLE);
extern int aGLSetClearColor(aHANDLE,COLORREF,float a=1.0);

extern int aGL2D(aHANDLE);
extern int aGL3D(aHANDLE);
extern int aGLSwitchTo(aHANDLE);

// LIGHTING
extern int aGLLighting(aHANDLE,int); // zapina a vypina praci se svetly
extern int aGLUseLight(aHANDLE C,int svetlo, bool on);
extern int aGLLightColor(aHANDLE C,int svetlo,
                             float ambr, float ambg, float ambb,
                             float difr, float difg, float difb,
                             float spcr, float spcg, float spcb);
extern int aGLOmniLight(aHANDLE C,int svetlo,float x,float y,float z);
extern int aGLDirectLight(aHANDLE C,int svetlo,float targetx, float targety, float targetz);
extern int aGLSpotLight(aHANDLE C,int svetlo,
                            float x,float y,float z,
                            float targetx,float targety,float targetz,
                            float cutoff, float exp
                          );
                      
// pozor, vliv ma i glColor !!!
__export int aGLMaterial(aHANDLE,
                           float ambr,float ambg,float ambb,float, // ambient
                           float difr,float difg,float difb,float, // difuse
                           float spcr,float spcg,float spcb,float, // specular
                           float emmr,float emmg,float emmb,float, // emisson
                           int shin
                          );

// objekty
extern int aGLDeleteObject(aHANDLE,aObjectHANDLE);

// FONTS
//  2D font, vyska,tucnost,kurziva,charset,rodina,jmeno pro tento kontext 
extern aFontHANDLE aGLCreateFont(aHANDLE,int,int,int,int,int,char*);
extern aFontHANDLE aGLCreateFontWidth(aHANDLE,int,int,int,int,int,char*,int); // sirka
extern aFontHANDLE aGLCreateFont(aHANDLE C,HFONT hFont);
extern int aGLPrintf(aHANDLE,aFontHANDLE,int,int,const char*,...);

//  3D font, vyska,tucnost,deviace,sirka,kurziva,charset,rodina,jmeno pro tento kontext
extern aFontHANDLE aGLCreate3DFont(aHANDLE,int,int,float,float,int,int,int,char*);
extern int aGL3DPrintf(aHANDLE,aFontHANDLE,const char*,...);

// IKONY - deprectated
//extern aIconHANDLE aGLCreateIcon(aHANDLE, HICON, int);
//extern int aGLDrawIcon(aHANDLE,aIconHANDLE,int,int);

// TEXTURY
extern aTextureHANDLE aGLCreateTexture(aHANDLE, aBitmapGL);
extern int aGLDrawTexture(aHANDLE C,aTextureHANDLE T,int,int,int,int); 

// KRESLENI
extern int aGLPoint(aHANDLE C,int,int,float size=1.0,COLORREF c=0xFF000000);
extern int aGLLine(aHANDLE C,int,int,int,int,float width=1.0,
                   COLORREF q=0xFF000000,COLORREF r=0xFF000000);
extern int aGLRect(aHANDLE C,int,int,int,int,float width=1.0,
                   COLORREF q=0xFF000000,COLORREF r=0xFF000000,
                   COLORREF s=0xFF000000,COLORREF t=0xFF000000);


#endif // __AGL_HPP__

