/*
  Name: aGL.cpp
  Copyright: Copyright(C) 2004, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aGL graphics library core
  
  This is not official version of the code.
*/

#include "aGL.hpp"

aGLContext::aGLContext() {
    hDC = NULL;
    hRC = NULL;
    sizex = 0;
    sizey = 0;
    
    nearplane = 0.0;
    farplane = 0.0;
    persp = 45.0;
}

aGLContext __aGL_internal_current_handle;

__export int aGLSwitchTo(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    if (__aGL_internal_current_handle.hDC != C->hDC ||
        __aGL_internal_current_handle.hRC != C->hRC) {
            
            __aGL_internal_current_handle.hDC = C->hDC;
            __aGL_internal_current_handle.hRC = C->hRC;
            
            wglMakeCurrent( C->hDC, C->hRC );
    }
    
    return 0;
}

__export aHANDLE aGLCreateContext (HWND hWnd, bool stencil, bool accum ) {
    aGLContext *C = new aGLContext;

    int iFormat;
    PIXELFORMATDESCRIPTOR pfd;
    C->hDC = GetDC(hWnd);

    ZeroMemory (&pfd, sizeof (pfd));
    pfd.nSize = sizeof (pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER |
                  PFD_SWAP_EXCHANGE;
    pfd.iPixelType = PFD_TYPE_RGBA;

	pfd.cColorBits = 32;
	pfd.cRedBits = 8;
	pfd.cRedShift = 16;
	pfd.cGreenBits = 8;
	pfd.cGreenShift = 8;
	pfd.cBlueBits = 8;
	pfd.cBlueShift = 0;
	pfd.cAlphaBits = 0;
	pfd.cAlphaShift = 0;
	if (accum) {
    	pfd.cAccumBits = 32;
    	pfd.cAccumRedBits = 8;
    	pfd.cAccumGreenBits = 8;
    	pfd.cAccumBlueBits = 8;
    	pfd.cAccumAlphaBits = 8;
	}
	
	pfd.cDepthBits = 24;
	if (stencil) pfd.cStencilBits = 8;

	pfd.cAuxBuffers = 0;
	pfd.iLayerType = PFD_MAIN_PLANE;
	pfd.bReserved = 0;
	pfd.dwLayerMask = 0;
	pfd.dwVisibleMask = 0;
	pfd.dwDamageMask = 0;
    
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat (C->hDC, &pfd);
    SetLastError(0);

    SetPixelFormat (C->hDC, iFormat, &pfd);

    C->hRC = wglCreateContext( C->hDC );

    aGLSwitchTo(C);
    glLoadIdentity();

    LOGBRUSH lbr;
    GetObject( (HGDIOBJ)GetClassLong(hWnd,GCL_HBRBACKGROUND), sizeof(lbr), &lbr );

    COLORREF crback = lbr.lbColor;
    glClearColor(GetRValue(crback)/255.0,
                 GetGValue(crback)/255.0, 
                 GetBValue(crback)/255.0, 1.0f);
    C->nearplane = 1.0;
    C->farplane = 64.0;
    
    return C;
}

__export int aGLDeleteContext (HWND hWnd, aHANDLE C){
    if (!C) return AGL_ERROR_BADPARAM;

    wglMakeCurrent (NULL, NULL);
    wglDeleteContext (C->hRC);
    ReleaseDC (hWnd, C->hDC);
    
    delete C;
    C = NULL;
    
    return 0;
}

__export int aGLResize(aHANDLE C,int x,int y) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    C->sizex = x;
    C->sizey = y;

    aGL2D(C);
    return 0;
};

__export int aGLBeginPaint(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    
    return 0;
}
__export int aGLEndPaint(HWND hwnd, aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    SwapBuffers(C->hDC);

    { HDC hDC;
      PAINTSTRUCT ps;
      hDC = BeginPaint( hwnd , &ps );
      EndPaint( hwnd, &ps ); }
      
//    Sleep(15);
    
    return 0;
}

int __aGL_pure_opengl_used = 0;
__export int aGLBeginPureOpenGL(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    if (__aGL_pure_opengl_used) return 1;

    __aGL_pure_opengl_used = 1;
    aGLSwitchTo(C);
    glLoadIdentity();

    return 0;
}
__export int aGLEndPureOpenGL(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    __aGL_pure_opengl_used = 0;
    return 0;
}


__export aFontHANDLE aGLCreateFont(aHANDLE C,int h,int bold,int it,
                                   int chs,int fam,char*fontname){
    return  aGLCreateFontWidth(C,h,bold,it,chs,fam,fontname,0);
}

__export aFontHANDLE aGLCreateFontWidth(aHANDLE C,int h,int bold,int it,
                                   int chs,int fam,char*fontname,int width){
    if (!C->hRC) return NULL;

    aGLFont *F = new aGLFont;
	HFONT font;

    aGLSwitchTo(C);
    glLoadIdentity();

	F->displaylist = glGenLists(256);
    F->count = 256;
    F->sizex = F->sizey = 0;

	font = CreateFont(h,width,0,0,bold,it,FALSE,FALSE,
					  chs,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,
	                  4,fam,fontname);
	HFONT hold = (HFONT)SelectObject(C->hDC, font);
	
	int s;
    GetCharWidth(C->hDC,'A','A',&s);
    F->param1 = s;
        
    int ret = wglUseFontBitmaps(C->hDC,0,256,F->displaylist);
    if (!ret) { // sometime XP behaviour fix
        wglUseFontBitmaps(C->hDC,0,256,F->displaylist);
    }

    SelectObject(C->hDC, hold);
	DeleteObject(font);

    return F;
}

__export aFontHANDLE aGLCreateFont(aHANDLE C,HFONT hFont){
    if (!C->hRC) return NULL;

    aGLFont *F = new aGLFont;

    aGLSwitchTo(C);
    glLoadIdentity();

	F->displaylist = glGenLists(256);
    F->count = 256;

	SelectObject(C->hDC, hFont);

	int s;
    GetCharWidth(C->hDC,'A','A',&s);
    F->param1 = s;

    wglUseFontBitmaps(C->hDC,0,256,F->displaylist);

    return F;
}

__export int aGLDeleteObject(aHANDLE C,aObjectHANDLE F) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    if (!F) return AGL_ERROR_BADPARAM;

    aGLSwitchTo(C);

    if (!glIsList(F->displaylist)) return AGL_ERROR_BADPARAM;
    glDeleteLists( F->displaylist ,F->count);
    
    delete F;
    F = NULL;
    return 0;
}

__export int aGLPrintf( aHANDLE C, aFontHANDLE F, int x, int y,
                        const char*fmt , ...) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    if (fmt == NULL) return -1;

    char text[2048];

    VA_OPEN (ap, fmt);
        VA_FIXEDARG (ap, const char *, fmt);
        vsprintf (text, fmt, ap);
    VA_CLOSE (ap); 
    
/*    va_list ap;
    va_start(ap, fmt);
    vsprintf(text, fmt, ap);
    va_end(ap);//*/

    aGLSwitchTo(C);
    glPushAttrib(GL_LIST_BIT);
    glListBase(F->displaylist);

    glRasterPos2i(x,y);
    glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
    glPopAttrib();
    
    return 0;
}

__export aFontHANDLE aGLCreate3DFont(aHANDLE C,int h,int bold,float q,float z,
                                     int it,int chs,int fam,char*fontname){
    if (!C->hRC) return NULL;

    aGLFont *F = new aGLFont;
	HFONT font;

    aGLSwitchTo(C);
    glLoadIdentity();
	
    F->displaylist = glGenLists(256);
    F->count = 256;
    
	font = CreateFont(h,0,0,0,bold,it,FALSE,FALSE,
					  chs,OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,
	                  DRAFT_QUALITY,fam,
					  fontname);
	SelectObject(C->hDC, font);
    wglUseFontOutlines(C->hDC,0,256,F->displaylist,q,z,WGL_FONT_POLYGONS,NULL);
	DeleteObject(font);

    return F;
}

__export int aGL3DPrintf(aHANDLE C,aFontHANDLE F,
                         const char*fmt,...) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    if (fmt == NULL) return -1;

    char text[2048];

    VA_OPEN (ap, fmt);
        VA_FIXEDARG (ap, const char *, fmt);
        vsprintf (text, fmt, ap);
    VA_CLOSE (ap); 

/*    va_list ap;
    va_start(ap, fmt);
    vsprintf(text, fmt, ap);
    va_end(ap);//*/

    aGLSwitchTo(C);
    glPushAttrib(GL_LIST_BIT);
    glListBase(F->displaylist);

    glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
    
    glPopAttrib();
    return 0;
}

__export int aGL2D(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glLoadIdentity();
    glViewport(0,0,C->sizex,C->sizey);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0,C->sizex,C->sizey,0,1,-1);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity(); 
    glDisable(GL_DEPTH_TEST);

    return 0;
}
__export int aGL3D(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glLoadIdentity();
    glViewport(0,0,C->sizex,C->sizey);
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	const GLdouble pi = 3.1415926535897932384626433832795;
	GLdouble fW, fH;
	
	fH = tan( C->persp / 180 * pi ) * C->nearplane / 2;
	fW = fH * (double)C->sizex/C->sizey;
	glFrustum( -fW, fW, -fH, fH, C->nearplane, C->farplane );

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity(); 
    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
    glEnable(GL_DEPTH_TEST);
    
    return 0;
}

__export int aGLLighting(aHANDLE C,int svetlo) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    if (svetlo) {
        glEnable(GL_LIGHTING);
        glEnable(GL_COLOR_MATERIAL);	 
    } else {
        glDisable(GL_LIGHTING);
        glDisable(GL_COLOR_MATERIAL);	 
    }
    return 0;
}

__export int aGLLightColor(aHANDLE C,int svetlo,
                             float ambr, float ambg, float ambb,
                             float difr, float difg, float difb,
                             float spcr, float spcg, float spcb
                          ) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    aGLSwitchTo(C);

    GLfloat LightAmbient[]={ ambr,ambg,ambb, 1.0 };
    GLfloat LightDiffuse[]={ difr,difg,difb, 1.0 };
    GLfloat LightSpecular[]={spcr,spcg,spcb, 1.0 };
        
    glLightfv(GL_LIGHT0+svetlo, GL_AMBIENT, LightAmbient);
    glLightfv(GL_LIGHT0+svetlo, GL_DIFFUSE, LightDiffuse);
    glLightfv(GL_LIGHT0+svetlo, GL_SPECULAR,LightSpecular);

    return 0;
}

__export int aGLUseLight(aHANDLE C,int svetlo, bool on) {
    aGLSwitchTo(C);
    if (on) glEnable(GL_LIGHT0+svetlo); else glDisable(GL_LIGHT0+svetlo);
    return 0;
}

__export int aGLOmniLight(aHANDLE C,int svetlo,
                            float x,float y,float z // pozice
                          ) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    aGLSwitchTo(C);

    GLfloat LightPosition[]={ x,y,z, 1.0 };
    glLightfv(GL_LIGHT0+svetlo, GL_POSITION,LightPosition);
    return 0;
}
__export int aGLDirectLight(aHANDLE C,int svetlo,
                            float targetx, float targety, float targetz // smer
                          ) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    aGLSwitchTo(C);

    GLfloat LightPosition[]={ targetx, targety, targetz, 0.0 };
    glLightfv(GL_LIGHT0+svetlo, GL_POSITION,LightPosition);
    return 0;
}

__export int aGLSpotLight(aHANDLE C,int svetlo,
                            float x,float y,float z, // pozice
                            float targetx,float targety,float targetz, // smer
                            float cutoff, float exp // uhel a koncentrace
                          ) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    aGLSwitchTo(C);

    GLfloat LightPosition[]={ x,y,z, 1.0 };
    GLfloat LightTarget[]={ targetx,targety,targetz, 0.0 };
    GLfloat LightCutoff = cutoff;
        
    glLightfv(GL_LIGHT0+svetlo, GL_POSITION, LightPosition);
    glLightfv(GL_LIGHT0+svetlo, GL_SPOT_DIRECTION, LightPosition);
    glLightf (GL_LIGHT0+svetlo, GL_SPOT_EXPONENT, exp);
    glLightf (GL_LIGHT0+svetlo, GL_SPOT_CUTOFF, cutoff);
    return 0;
}

__export int aGLSpotAttenuation(aHANDLE C,int svetlo, int cons, int lin, int quad) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    aGLSwitchTo(C);

    glLightf (GL_LIGHT0+svetlo, GL_CONSTANT_ATTENUATION , cons);
    glLightf (GL_LIGHT0+svetlo, GL_LINEAR_ATTENUATION , lin);
    glLightf (GL_LIGHT0+svetlo, GL_QUADRATIC_ATTENUATION , quad);

    return 0;
}

__export int aGLMaterial(aHANDLE C,
                           float ambr,float ambg,float ambb,float amba, // ambient
                           float difr,float difg,float difb,float difa, // difuse
                           float spcr,float spcg,float spcb,float spca, // specular
                           float emmr,float emmg,float emmb,float emma, // emisson
                           int shin // shinines
                          ) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    GLfloat MatAmbient[]={ ambr,ambg,ambb, amba };
    GLfloat MatDiffuse[]={ difr,difg,difb, difa };
    GLfloat MatSpecular[]={spcr,spcg,spcb, spca };
    GLfloat MatEmission[]={emmr,emmg,emmb, emma };

    glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,MatAmbient);
    glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,MatDiffuse);
    glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,MatSpecular);
    glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,MatEmission);
    glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,shin);

    return 0;                          
}

__export aTextureHANDLE aGLCreateTexture(aHANDLE C, aBitmapGL B) {
    if (!C->hRC) return NULL;

    aGLSwitchTo(C);
    aGLTexture *T = new aGLTexture;
    
    T->displaylist = B.getOpenGLDisplayList();
    T->count = 1;
    T->sizex = B.getwidth();
    T->sizey = B.getheight();
        
    return T;
};

__export int aGLDrawTexture(aHANDLE C,aTextureHANDLE T,int x,int y,int w,int h) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    if (!T) return AGL_ERROR_BADPARAM;
    aGLSwitchTo(C);

    if (w==0) w = T->sizex; 
    if (h==0) h = T->sizey;

    glTranslatef(x,y,0);
    glScalef(w,h,1);
    glCallList( T->displaylist );
    return 0;
}

__export int aGLPoint(aHANDLE C,int x,int y,float size,COLORREF c) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glPushAttrib(GL_CURRENT_BIT|GL_ENABLE_BIT|GL_POINT_BIT);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable( GL_POINT_SMOOTH );
    glEnable( GL_BLEND );
    glPointSize(size);

    glBegin(GL_POINTS);
        if (c!=0xFF000000) glColor3f( GetRValue(c)/255.0,
                                      GetGValue(c)/255.0,GetBValue(c)/255.0 );
        glVertex2i(x,y);
    glEnd();    
    glPopAttrib();

    return 0;
}
__export int aGLLine(aHANDLE C,int a,int b,int c,int d,float width,
                     COLORREF q,COLORREF r) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glPushAttrib(GL_CURRENT_BIT|GL_ENABLE_BIT|GL_COLOR_BUFFER_BIT|GL_LINE_BIT);
    if (width!=1.0) glEnable( GL_LINE_SMOOTH );
    glLineWidth(width);

    glBegin(GL_LINES);
        if (q!=0xFF000000)
            glColor3f( GetRValue(q)/255.0,GetGValue(q)/255.0,GetBValue(q)/255.0 );
        glVertex2i(a,b);
        if (r!=0xFF000000)
            glColor3f( GetRValue(r)/255.0,GetGValue(r)/255.0,GetBValue(r)/255.0 );
        glVertex2i(c,d);
    glEnd();    
    glPopAttrib();

    return 0;
}
__export int aGLRect(aHANDLE C,int a,int b,int c,int d,float width,
                     COLORREF q,COLORREF r,
                     COLORREF s,COLORREF t){
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glPushAttrib(GL_CURRENT_BIT|GL_ENABLE_BIT|GL_COLOR_BUFFER_BIT|GL_LINE_BIT);
    if (width!=1.0) glEnable( GL_LINE_SMOOTH );
    glLineWidth(width);

    glBegin(GL_LINE_LOOP);
        if (q!=0xFF000000)
            glColor3f( GetRValue(q)/255.0,GetGValue(q)/255.0,GetBValue(q)/255.0 );
        glVertex2i(a,b);
        if (r!=0xFF000000)
            glColor3f( GetRValue(r)/255.0,GetGValue(r)/255.0,GetBValue(r)/255.0 );
        glVertex2i(c,b);
        if (s!=0xFF000000)
            glColor3f( GetRValue(s)/255.0,GetGValue(s)/255.0,GetBValue(s)/255.0 );
        glVertex2i(c,d);
        if (t!=0xFF000000)
            glColor3f( GetRValue(t)/255.0,GetGValue(t)/255.0,GetBValue(t)/255.0 );
        glVertex2i(a,d);
    glEnd();    
    glPopAttrib();

    return 0;
}

__export int aGLClearBuffers(aHANDLE C) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;

    aGLSwitchTo(C);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    return 0;
}

__export int aGLSetClearColor(aHANDLE C,COLORREF c,float a) {
    if (!C->hRC) return AGL_ERROR_NOTINIT;
    if (a>1.0 || a<0.0) a=1.0;

    aGLSwitchTo(C);
    glClearColor(GetRValue(c)/255.0,
                 GetGValue(c)/255.0, 
                 GetBValue(c)/255.0, a);
    return 0;
}

