/*
  Name: aModel.cpp
  Copyright: Copyright(C) 2003, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aModel format class
  
  This is not official version of the code.
*/

#include "aModel.hpp"
#include <ctype.h>

aModel::aModel(){
    strcpy(magic,"aModel\0\0");

    n_vertex = 0;
    n_face = 0;
    n_dummy = 0;
    n_material = 0;
    
    vertex = NULL;
    face = NULL;
    dummy = NULL;
    material = NULL;
    
    p1 = 0;
    p2 = 0;
}

aModel::~aModel() {
    if (vertex) GlobalFree(vertex);
    if (face) GlobalFree(face);
    if (dummy) GlobalFree(dummy);
    if (material) GlobalFree(material);
}

int aModel::load(char *name){
    FILE *f;
    f = fopen(name,"rb");
    if (!f) return AGL_ERROR_FILEERR;
    
    // testnout magic
    char temp[8];
    fread(temp,8,1,f);
    if (strcmp(magic,temp)){ fclose(f); return AGL_ERROR_BADFILE; }
    
    // vycistit
    if (vertex) GlobalFree(vertex);
    if (face) GlobalFree(face);
    if (dummy) GlobalFree(dummy);
    if (material) GlobalFree(material);

    // nacist zacatek hlavicky
    fseek(f,0,SEEK_SET); // reset souborku
    fread(this,sizeof(aModel)-(sizeof(vertex)+sizeof(face)
                               +sizeof(dummy)+sizeof(material)),1,f);
    if (n_vertex) {
        vertex = (__vertex*) GlobalAlloc(GPTR, sizeof(__vertex) * n_vertex );
        if (!vertex) return AGL_ERROR_MEMORY;
    }
    if (n_face) {
        face = (__face*) GlobalAlloc(GPTR, sizeof(__face) * n_face );
        if (!vertex) return AGL_ERROR_MEMORY;
    }
    if (n_dummy) {
        dummy = (__dummy*) GlobalAlloc(GPTR, sizeof(__dummy) * n_dummy );
        if (!vertex) return AGL_ERROR_MEMORY;
    }
    if (n_material) {
        material = (__material*) GlobalAlloc(GPTR, sizeof(__material) * n_material );
        if (!vertex) return AGL_ERROR_MEMORY;
    }

    if (p1 > 1){ fclose(f); return AGL_ERROR_COMPRESSION; } // nonsupported compression

    if (p1 == 0){
        if (n_vertex)   fread(vertex,sizeof(__vertex),n_vertex,f);
        if (n_face)     fread(face,sizeof(__face),n_face,f);
        if (n_dummy)    fread(dummy,sizeof(__dummy),n_dummy,f);
        if (n_material) fread(material,sizeof(__material),n_material,f);
    }
    if (p1 == 1) {
        unsigned long insize = filesize(f) - 32;/*
                              (  sizeof(aModel)-(sizeof(vertex)+sizeof(face)
                                +sizeof(dummy)+sizeof(material) ) );*/
        char *in = (char*)GlobalAlloc(GPTR, insize );
        unsigned long destLen = sizeof(__vertex)*n_vertex + sizeof(__face)*n_face +
                                sizeof(__dummy)*n_dummy + sizeof(__material)*n_material;

        char *tmp = (char*)GlobalAlloc(GPTR, destLen );
        if (!in) return AGL_ERROR_MEMORY; // not enough memory
        if (!tmp) return AGL_ERROR_MEMORY; // not enough memory
        
        fread(in,insize,1,f);
        
        int ret = uncompress( (Bytef*)tmp, &destLen, (Bytef*)in, insize );
        
        if (ret == Z_MEM_ERROR){ GlobalFree(in);GlobalFree(tmp);return AGL_ERROR_MEMORY;}
        if (ret == Z_BUF_ERROR){ GlobalFree(in);GlobalFree(tmp);return AGL_ERROR_BUFFER;}
        if (ret == Z_DATA_ERROR){ GlobalFree(in);GlobalFree(tmp);return AGL_ERROR_ZDATA;}
        
        CopyMemory(vertex,  tmp,sizeof(__vertex)*n_vertex);
        CopyMemory(face,    &tmp[sizeof(__vertex)*n_vertex],  sizeof(__face)*n_face);
        CopyMemory(dummy,   &tmp[sizeof(__face)*n_face+sizeof(__vertex)*n_vertex],
                            sizeof(__dummy)*n_dummy);
        CopyMemory(material,&tmp[sizeof(__dummy)*n_dummy+sizeof(__face)*n_face+
                            sizeof(__vertex)*n_vertex],sizeof(__material)*n_material);
        
        GlobalFree(in);
        GlobalFree(tmp);
    }

    fclose(f);
    return 0;
} // load

int aModel::save(char *name){
    FILE *f;
    f = fopen(name,"wb");
    if (!f) return AGL_ERROR_FILEERR;
    
    p1 = 0;
    fwrite(this,sizeof(aModel)-(sizeof(vertex)+sizeof(face)
                               +sizeof(dummy)+sizeof(material) ),1,f); // hlavicku
    if (n_vertex)   fwrite(vertex,sizeof(__vertex),n_vertex,f);
    if (n_face)     fwrite(face,sizeof(__face),n_face,f);
    if (n_dummy)    fwrite(dummy,sizeof(__dummy),n_dummy,f);
    if (n_material) fwrite(material,sizeof(__material),n_material,f);
    
    fclose(f);
    return 0;
} // save

int aModel::save_c(char *name){
    FILE *f;
    f = fopen(name,"wb");
    if (!f) return AGL_ERROR_FILEERR;
    
    p1 = 1;
    fwrite(this,sizeof(aModel)-(sizeof(vertex)+sizeof(face)
                               +sizeof(dummy)+sizeof(material) ),1,f); // hlavicku

    unsigned long inLen = (sizeof(__vertex)*n_vertex
                           +sizeof(__face)*n_face+sizeof(__dummy)*n_dummy
                           +sizeof(__material)*n_material);
    unsigned long outLen = (int)(1.001*(float)inLen)+32;
    
    char *out = (char*)GlobalAlloc(GPTR,outLen);
    if (!out) return AGL_ERROR_MEMORY;
    char *in = (char*)GlobalAlloc(GPTR,inLen);
    if (!in){ GlobalFree(out); return AGL_ERROR_MEMORY;}
    
    CopyMemory(in, vertex, sizeof(__vertex)*n_vertex);
    CopyMemory(&in[sizeof(__vertex)*n_vertex], face, sizeof(__face)*n_face);
    CopyMemory(&in[sizeof(__face)*n_face+sizeof(__vertex)*n_vertex],
                   dummy,    sizeof(__dummy)*n_dummy);
    CopyMemory(&in[sizeof(__dummy)*n_dummy+sizeof(__face)*n_face+sizeof(__vertex)*n_vertex],
                   material, sizeof(__material)*n_material);
    
    int ret = compress2( (Bytef*)out, &outLen,
                         (Bytef*)in,   inLen,  9 );
    
    if (ret == Z_MEM_ERROR){ GlobalFree(out);GlobalFree(in); return AGL_ERROR_MEMORY; }
    if (ret == Z_BUF_ERROR){ GlobalFree(out);GlobalFree(in); return AGL_ERROR_BUFFER; }
    if (ret==Z_STREAM_ERROR){ GlobalFree(out);GlobalFree(in);return AGL_ERROR_ZSTREAM;}
   
    fwrite(out,outLen,1,f); // data
    GlobalFree(out);
    GlobalFree(in);

    fclose(f);
    return 0;
} // save


int aModel::import_asc(char *name){
    char szNamedObj[] = "Named object:";
    char szTriMesh[] = "Tri-mesh";
    char szVertexList[] = "Vertex list:";
    char szFaceList[] = "Face list:";

    FILE *f;
    f = fopen(name,"rb");
    if (!f) return AGL_ERROR_FILEERR;

    unsigned int i,tmp;
    char radek[2048];
    do { fgets(radek,2048,f); } while(strncmp(szNamedObj,radek,strlen(szNamedObj)));
    do { fgets(radek,2048,f); } while(strncmp(szTriMesh,radek,strlen(szTriMesh)));

    sscanf(radek,"Tri-mesh, Vertices: %d     Faces: %d",&n_vertex,&n_face);
    if (vertex) GlobalFree(vertex);
    if (n_vertex) {
        vertex = (__vertex*) GlobalAlloc(GPTR, sizeof(__vertex) * n_vertex );
        if (!vertex) return AGL_ERROR_MEMORY;
    }
    if (face) GlobalFree(face);
    if (n_face) {
        face = (__face*) GlobalAlloc(GPTR, sizeof(__face) * n_face );
        if (!vertex) return AGL_ERROR_MEMORY;
    }
    
    do { fgets(radek,2048,f); } while(strncmp(szVertexList,radek,strlen(szVertexList)));
    for (i=0;i<n_vertex;) {
         fgets(radek,2048,f);
         if (!isalnum(radek[0])) continue; // neni nic
         sscanf(radek,"Vertex %d:  X: %f     Y: %f     Z: %f",
                              &tmp, &vertex[i].x, &vertex[i].y, &vertex[i].z);
         i++;
    }
    do { fgets(radek,2048,f); } while(strncmp(szFaceList,radek,strlen(szFaceList)));
    for (i=0;i<n_face;) {
         fgets(radek,2048,f);
         if (!isalnum(radek[0])) continue; // neni nic
         
         if (!strncmp("Face",radek,4)) {
             sscanf(radek,"Face %d:    A: %d B: %d C: %d",
                                &tmp, &face[i].v[0], &face[i].v[1], &face[i].v[2]);
             i++;
         }
    }
    
    fclose(f);
    
    if (!material) {
         float amb[4] = { 0.1,0.1,0.1,1.0 };
         float dif[4] = { 0.5,0.5,0.5,1.0 };
         float spe[4] = { 1.0,1.0,1.0,1.0 };
         float emi[4] = { 0.0,0.0,0.0,1.0 };
         float shi = 64.0;
         addmaterial("Default",amb,dif,spe,emi,shi);
    }

    return 0;
}

int aModel::import_asc_adv(char *name){
    char szNamedObj[] = "Named object:";
    char szTriMesh[] = "Tri-mesh";
    char szVertexList[] = "Vertex list:";
    char szFaceList[] = "Face list:";

    FILE *f;
    f = fopen(name,"rt");
    if (!f) return AGL_ERROR_FILEERR;

    unsigned int i,tmp;
    char radek[2048];
    
    if (vertex) GlobalFree(vertex); vertex=NULL;
    if (face) GlobalFree(face); face=NULL;
    if (material) GlobalFree(material); material=NULL;

    n_vertex = n_face = n_material = 0;
    unsigned int t_vertex = 0, t_face = 0;

    {
         float amb[4] = { 0.1,0.1,0.1,1.0 };
         float dif[4] = { 0.5,0.5,0.5,1.0 };
         float spe[4] = { 1.0,1.0,1.0,1.0 };
         float emi[4] = { 0.0,0.0,0.0,1.0 };
         float shi = 64.0;
         addmaterial("Default",amb,dif,spe,emi,shi);
    }

//    FILE *d = fopen("debug.txt","wt");

    do {
        // look for begining
        do { fgets(radek,2048,f); if (feof(f)) goto iaa_end;} 
            while(strncmp(szNamedObj,radek,strlen(szNamedObj)));
        do { fgets(radek,2048,f); if (feof(f)) goto iaa_end;} 
            while(strncmp(szTriMesh,radek,strlen(szTriMesh)));
    
        sscanf(radek,"Tri-mesh, Vertices: %d     Faces: %d",&t_vertex,&t_face);
        if (t_vertex) {
            // realloc
            void *ptr = GlobalAlloc(GPTR, sizeof(__vertex) * (n_vertex + t_vertex) );
            if (!ptr) return AGL_ERROR_MEMORY;
            
            CopyMemory(ptr,vertex,sizeof(__vertex) * n_vertex);
            GlobalFree(vertex);
            
            vertex = (__vertex*)ptr;
//            fprintf(d,"vertex: new: %d, old: %d\n",t_vertex,n_vertex);
        }
        if (t_face) {
            void *ptr = GlobalAlloc(GPTR, sizeof(__face) * (n_face + t_face) );
            if (!ptr) return AGL_ERROR_MEMORY;
            
            CopyMemory(ptr,face,sizeof(__face) * n_face);
            GlobalFree(face);
            
            face = (__face*)ptr;
//            fprintf(d,"faces: new: %d, old: %d\n",t_face,n_face);
        }
        
        do { fgets(radek,2048,f); if (feof(f)) goto iaa_end;} 
            while(strncmp(szVertexList,radek,strlen(szVertexList)));
//        fprintf(d,"vertices loading: ");
        for (i=0;i<t_vertex;) {
             fgets(radek,2048,f);
             if (!isalnum(radek[0])) continue; // neni nic
             sscanf(radek,"Vertex %d:  X: %f     Y: %f     Z: %f",
                                  &tmp, &vertex[n_vertex+i].x,
                                  &vertex[n_vertex+i].y, &vertex[n_vertex+i].z);
             i++;
//             fprintf(d,".");
        }
        do { fgets(radek,2048,f); if (feof(f)) goto iaa_end;} 
            while(strncmp(szFaceList,radek,strlen(szFaceList)));
//        fprintf(d,"faces loading:\n");
        for (i=0;;) {
             fgets(radek,2048,f);
             if (!isalnum(radek[0])) continue; // neni nic
             
             if (!strncmp("Face",radek,4)) {
                 sscanf(radek,"Face %d:    A: %d B: %d C: %d",
                                    &tmp, &face[n_face+i].v[0],
                                    &face[n_face+i].v[1], &face[n_face+i].v[2]);
                 face[n_face+i].v[0] += n_vertex;
                 face[n_face+i].v[1] += n_vertex;
                 face[n_face+i].v[2] += n_vertex;
                 
                 i++;
//                 fprintf(d,"    got new face\n");
//                 fflush(d);
             }
             
             if (!strncmp("Material",radek,8)) {
                 char aaa[256];
//                 sscanf(radek,"Material:\"%s\"",aaa);
                 for (int tmp=0;tmp<11;tmp++) {
                     int c = radek[10+tmp];
                     if (c == '\"'){ aaa[tmp] = 0; break; }
                     aaa[tmp] = c;
                 }
//                 for (int tmp=0;;tmp++) if (fgetc(f) == '\n' || feof(f)) break;
                                  
//                 fprintf(d,"    got new material %s\n",aaa);
                 // najit material
                 int exist = -1;
                 for (int k=0;k<getnmaterial();k++) {
                     char name[12];
                     getmaterialname(k,name);
                     if (!strcmp(name,aaa)) {
                         exist = k;
//                         fprintf(d,"        %s == %s [%d]\n",name,aaa,k);
                         break;
                     }
                 }
//                 fprintf(d,"        exist: %d\n",exist);
                 
                 if (exist == -1) { // new material
                     float amb[4] = { 0.1,0.1,0.1,1.0 };
                     float dif[4] = { 0.5,0.5,0.5,1.0 };
                     float spe[4] = { 1.0,1.0,1.0,1.0 };
                     float emi[4] = { 0.0,0.0,0.0,1.0 };
                     float shi = 64.0;
                     
//                     fprintf(d,"        adding\n");
                     addmaterial(aaa,amb,dif,spe,emi,shi);
                     face[n_face+i-1].m = getnmaterial()-1;
/*                     
                     FILE *f = fopen("test.out","at");
                     fprintf(f,"new %d: %d\n",n_face+i-1,face[n_face+i-1].m);
                     fclose(f);//*/
                 } else { // already exist
//                     fprintf(d,"        exist\n");
                     face[n_face+i-1].m = exist;
/*
                     FILE *f = fopen("test.out","at");
                     fprintf(f,"exist %d: %d\n",n_face+i-1,face[n_face+i-1].m);
                     fclose(f);//*/
                 }
//                 fflush(d);
//                 Sleep(1);
                 if (i>=t_face) break;
             }
        } // face
        
        n_vertex += t_vertex;
        n_face += t_face;
    } while(!feof(f));
    iaa_end:
    
    fclose(f);
//    fclose(d);
    return 0;
}


inline int aModel::getnvertex(){
    return n_vertex;
}
inline int aModel::getnface(){
    return n_face;
}
inline int aModel::getndummy(){
    return n_dummy;
}
inline int aModel::getnmaterial(){
    return n_material;
}

int aModel::setnvertex(int n){
    if (n<0) return AGL_ERROR_BADPARAM;

    void *x = GlobalAlloc(GPTR, sizeof(__vertex) * n );
    if (!x) return AGL_ERROR_MEMORY;
    CopyMemory(x, vertex, sizeof(__vertex) * n_vertex);
    GlobalFree(vertex);

    vertex = (__vertex*)x;
    n_vertex = n;

    return 0;
}
int aModel::setnface(int n){
    if (n<0) return AGL_ERROR_BADPARAM;

    void *x = GlobalAlloc(GPTR, sizeof(__face) * n );
    if (!x) return AGL_ERROR_MEMORY;
    CopyMemory(x, face, sizeof(__face) * n_face);
    GlobalFree(face);

    face = (__face*)x;
    n_face = n;
    return 0;
}
int aModel::setndummy(int n){
    if (n<0) return AGL_ERROR_BADPARAM;

    void *x = GlobalAlloc(GPTR, sizeof(__dummy) * n );
    if (!x) return AGL_ERROR_MEMORY;
    CopyMemory(x, dummy, sizeof(__dummy) * n_dummy);
    GlobalFree(dummy);
    
    dummy = (__dummy*)x;
    n_dummy = n;
    return 0;
}
int aModel::setnmaterial(int n){
    if (n<0) return AGL_ERROR_BADPARAM;

    void *x = GlobalAlloc(GPTR, sizeof(__material) * n );
    if (!x) return AGL_ERROR_MEMORY;
    CopyMemory(x, material, sizeof(__material) * n_material);
    GlobalFree(material);

    material = (__material*)x;
    n_material = n;
    return 0;
}

int aModel::addvertex(float x,float y,float z,float w){
    int err = 0;
    
    int n = getnvertex();

    err = setnvertex(n+1);
    if (err) return err;
    
    err = setvertex(n,x,y,z,w);
    if (err) return err;

    return 0;
}
int aModel::setvertex(unsigned long n,float x,float y,float z,float w){
    if (n<0 || n>=getnvertex()) return AGL_ERROR_BADPARAM;
    vertex[n].x = x;
    vertex[n].y = y;
    vertex[n].z = z;
    vertex[n].w = w;
    return 0;
}

int aModel::addface(unsigned long a,unsigned long b,unsigned long c,unsigned long m){
    int err = 0;
    
    int n = getnface();
    err = setnface(n+1);
    if (err) return err;
    
    err = setface(n,a,b,c,m);
    if (err) return err;

    return 0;
}
int aModel::setface(unsigned long n, unsigned long a,unsigned long b,
                                     unsigned long c,unsigned long m){
    if (n<0 || n>=getnface()) return AGL_ERROR_BADPARAM;
    face[n].v[0] = a;
    face[n].v[1] = b;
    face[n].v[2] = c;
    face[n].m = m;
    return 0;
}

int aModel::setfacetexturebindingpoints(unsigned int n, float*tt){ 
    if (n<0 || n>=getnface()) return AGL_ERROR_BADPARAM;
    face[n].tbp[0] = tt[0];
    face[n].tbp[1] = tt[1];
    face[n].tbp[2] = tt[2];
    face[n].tbp[3] = tt[3];
    face[n].tbp[4] = tt[4];
    face[n].tbp[5] = tt[5];
    return 0;
}

int aModel::getfacetexturebindingpoints(unsigned int n, float*tt){ 
    if (n<0 || n>=getnface()) return AGL_ERROR_BADPARAM;
    tt[0] = face[n].tbp[0];
    tt[1] = face[n].tbp[1];
    tt[2] = face[n].tbp[2];
    tt[3] = face[n].tbp[3];
    tt[4] = face[n].tbp[4];
    tt[5] = face[n].tbp[5];
    return 0;
}

int aModel::adddummy(unsigned long id,float x,float y,float z){
    int err = 0;
    
    int n = getndummy();
    err = setndummy(n+1);
    if (err) return err;
    
    err = setdummy(n,id,x,y,z);
    if (err) return err;

    return 0;
}
int aModel::setdummy(unsigned long n,unsigned long id,float x,float y,float z){
    if (n<0 || n>=getndummy()) return AGL_ERROR_BADPARAM;
    dummy[n].id = id;
    dummy[n].x = x;
    dummy[n].y = y;
    dummy[n].z = z;
    return 0;
}

int aModel::addmaterial(char*name,float*amb,float*dif,float*spe,float*emi,float shin){
    int err = 0;
    
    int n = getnmaterial();
    err = setnmaterial(n+1);
    if (err) return err;
    
    err = setmaterial(n,name,amb,dif,spe,emi,shin);
    if (err) return err;

    return 0;
}
int aModel::setmaterial(unsigned long n,char*name,float*amb,float*dif,
                        float*spe,float*emi,float s){
    if (n<0 || n>=getnmaterial()) return AGL_ERROR_BADPARAM;
    strncpy( material[n].name, name, 11 );
    material[n].name[11] = 0;
    
    memcpy( &material[n].ambient, amb, 4 * sizeof(float) );
    memcpy( &material[n].diffuse, dif, 4 * sizeof(float) );
    memcpy( &material[n].spectacular, spe, 4 * sizeof(float) );
    memcpy( &material[n].emission, emi, 4 * sizeof(float) );
    
    material[n].shininess = s;
    return 0;
}
int aModel::setmaterialtexture(unsigned long n,unsigned int texture){
    if (n<0 || n>=getnmaterial()) return AGL_ERROR_BADPARAM;

    material[n].texture = texture;
    return 0;
}
int aModel::setmaterialname(unsigned long n,char*name){
    if (n<0 || n>=getnmaterial()) return AGL_ERROR_BADPARAM;
    memset( material[n].name,0,12 );
    strncpy( material[n].name, name, 11);
    return 0;
}
int aModel::getmaterialname(unsigned long n,char*name){
    if (n<0 || n>=getnmaterial()) return AGL_ERROR_BADPARAM;

    strcpy( name, material[n].name );
    return 0;
}


int aModel::getmaterialtexture(unsigned long n){
    if (n<0 || n>=getnmaterial()) return AGL_ERROR_BADPARAM;
    
    return material[n].texture;
}

int aModel::getvertex(int n,float *x,float *y,float *z) {
    if (n<0 || n>=getnvertex()) return AGL_ERROR_BADPARAM;
    *x = vertex[n].x;
    *y = vertex[n].y;
    *z = vertex[n].z;
    return 0;
}
int aModel::getvertexusage(int n,unsigned int *arr,int len) {
    if (n<0 || n>=getnvertex()) return AGL_ERROR_BADPARAM;
    int j=0;
    for (unsigned int i=0;i<getnface();i++) {
        if (face[i].v[0] == n || face[i].v[1] == n || face[i].v[2] == n) {
            arr[j] = i;
            j++;
            if (j==len) return j;
        }            
    }
    return j;
}

int aModel::getdummy(int n,unsigned int *id,float *x,float *y,float *z) {
    if (n<0 || n>=getndummy()) return AGL_ERROR_BADPARAM;
    *x = dummy[n].x;
    *y = dummy[n].y;
    *z = dummy[n].z;
    *id = dummy[n].id;
    return 0;
}
int aModel::getmaterial(int n,float *amb,float *dif,float *spe,float*emi,float *shi) {
    if (n<0 || n>=getnmaterial()) return AGL_ERROR_BADPARAM;
    amb[0] = material[n].ambient[0];
    amb[1] = material[n].ambient[1];
    amb[2] = material[n].ambient[2];
    amb[3] = material[n].ambient[3];

    dif[0] = material[n].diffuse[0];
    dif[1] = material[n].diffuse[1];
    dif[2] = material[n].diffuse[2];
    dif[3] = material[n].diffuse[3];

    spe[0] = material[n].spectacular[0];
    spe[1] = material[n].spectacular[1];
    spe[2] = material[n].spectacular[2];
    spe[3] = material[n].spectacular[3];

    emi[0] = material[n].emission[0];
    emi[1] = material[n].emission[1];
    emi[2] = material[n].emission[2];
    emi[3] = material[n].emission[3];

    *shi = material[n].shininess;    
    return 0;
}

int aModel::isdummyid(unsigned int id) {
    for (int i=0;i<getndummy();i++) {
        if ( dummy[i].id == id ) { return 1; }    
    }
    return 0;
}

int aModel::getface(int n,unsigned int*a,unsigned int*b,unsigned int*c,unsigned int*m){
    if (n<0 || n>=getnface()) return AGL_ERROR_BADPARAM;
    *a = face[n].v[0];
    *b = face[n].v[1];
    *c = face[n].v[2];
    *m = face[n].m;
    return 0;
}

int aModel::glDraw() {
    unsigned int m_cache = 0xFFFFFFFF;
    
    for(int i=0;i<n_face;i++) {
      glBegin(GL_TRIANGLES);
        VECTOR3D vector[3];
        VECTOR3D normal;
        
        vector[0].x = vertex[ face[i].v[0] ].x;
        vector[0].y = vertex[ face[i].v[0] ].y;
        vector[0].z = vertex[ face[i].v[0] ].z;
        vector[1].x = vertex[ face[i].v[1] ].x;
        vector[1].y = vertex[ face[i].v[1] ].y;
        vector[1].z = vertex[ face[i].v[1] ].z;
        vector[2].x = vertex[ face[i].v[2] ].x;
        vector[2].y = vertex[ face[i].v[2] ].y;
        vector[2].z = vertex[ face[i].v[2] ].z;
        
        normal = Normal(vector);
        normal.x = -normal.x;
        normal.y = -normal.y;
        normal.z = -normal.z;
        
        if (face[i].m >= 0 && face[i].m < getnmaterial()) {
            unsigned int m = face[i].m;
            if (m != m_cache) { // optimize materials
                m_cache = m;
                
                glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,material[m].ambient);
                glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,material[m].diffuse);
                glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,material[m].spectacular);
                glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,material[m].emission);
                glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,material[m].shininess);
            }
        }

        glNormal3fv( (GLfloat*)&normal );
        glVertex3fv( (GLfloat*)&vector[0] );
        glVertex3fv( (GLfloat*)&vector[1] );
        glVertex3fv( (GLfloat*)&vector[2] );
      glEnd();
    }
    return 0;
}

int aModel::getOpenGLDisplayList() {
    int list = glGenLists(1);
    glLoadIdentity();
    glNewList(list,GL_COMPILE);

    glDraw();

    glEndList();
    return list;
}

int aModel::translate(float x,float y,float z) {
    for(unsigned int i=0;i<n_vertex;i++) {
        vertex[i].x += x;
        vertex[i].y += y;
        vertex[i].z += z;
    }
    return 0;
}

int aModel::getboundingrect(float*x1,float*x2,float*y1,float*y2,float*z1,float*z2) {
    *z1 = *y1 = *x1 = 1000000.0f;
    *z2 = *y2 = *x2 = -*x1; 
    
    for(unsigned int i=0;i<n_vertex;i++) {
        if (vertex[i].x < *x1) *x1 = vertex[i].x;
        if (vertex[i].x > *x2) *x2 = vertex[i].x;
        
        if (vertex[i].y < *y1) *y1 = vertex[i].y;
        if (vertex[i].y > *y2) *y2 = vertex[i].y;

        if (vertex[i].z < *z1) *z1 = vertex[i].z;
        if (vertex[i].z > *z2) *z2 = vertex[i].z;
    }
    return 0;
}

int aModel::flipaxis_yz() {
    float tmp;
    for(unsigned int i=0;i<n_vertex;i++) {
        tmp = vertex[i].y;
        vertex[i].y = vertex[i].z;
        vertex[i].z = tmp;
    }
    return 0;
}




