/*
  Name: aModel.hpp
  Copyright: Copyright(C) 2003, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aModel format class

  Requirements: Windows Environment, Zlib compression library
  Handling: aModel model based functions
            Support 3D routines
            OpenGL support
  
  Comment: This version of aModel won't be probably compatible
           with future aModels.
  
  This is not official version of the code.
*/
#ifndef __AMODEL_HPP__
#define __AMODEL_HPP__

#include <windows.h>
#include <stdio.h>
#include <gl\gl.h>
#include <gl\glu.h>
//#include <gl\glext.h>
#include <zlib.h>

#include "a3DMath.hpp"
#include "aDef.hpp"

class aModel {
    private:
        char magic[8];
        unsigned long n_vertex; // number of vetices
        unsigned long n_face; // number of faces
        
        long p1; // compression type [0-pure data, 1-zlib]
        long p2; // reserved
        
        unsigned long n_dummy; // number of dummies
        unsigned long n_material; // number of materials
        
        struct __vertex {
            float x,y,z,w; // w - reserved
        } *vertex;
        
        struct __face {
            unsigned long v[3];
            unsigned long m; // assigned material
            // texture lock coords
            float tbp[6]; // texture bind points
            unsigned long reserved[2];
        } *face;
        
        struct __dummy {
            unsigned long id; 
            float x,y,z;
        } *dummy;       
        
        struct __material { // current 84 bytes length
            char name[12]; // 11 chars
        
            float ambient[4];
            float diffuse[4];
            float spectacular[4];
            float emission[4];
            float shininess; // 0 - 128

            unsigned int texture; // space for engine to assign texture from name
            
            char reserved[44];
        } *material;

    public:
        aModel();
        ~aModel();
        int load(char *name); 
        int save(char *name);
        int save_c(char *name); // saves compressed
        
        inline int getnvertex();
        inline int getnface();
        inline int getndummy();
        inline int getnmaterial();
        int setnvertex(int);
        int setnface(int);
        int setndummy(int);
        int setnmaterial(int);
        
        int addvertex(float x,float y,float z,float w=0.0);
        int setvertex(unsigned long n,float x,float y,float z,float w=0.0);
        
        int addface(unsigned long a,unsigned long b,unsigned long c,unsigned long m);
        int setface(unsigned long n, unsigned long a,unsigned long b,
                                     unsigned long c,unsigned long m);
        int getface(int n,unsigned int*a,unsigned int*b,unsigned int*c,unsigned int*m);
        int setfacetexturebindingpoints(unsigned int n,float*arr);
        int getfacetexturebindingpoints(unsigned int n,float*arr);
                                     
        int adddummy(unsigned long id,float x,float y,float z);
        int setdummy(unsigned long n,unsigned long id,float x,float y,float z);
        int getdummy(int n,unsigned int *id,float *x,float *y,float *z);
        int isdummyid(unsigned int); // TODO: Rename to existdummyid or something
        
        int addmaterial(char*name,float*amb,float*dif,float*spe,float*emi,float shin);
        int setmaterial(unsigned long n,char*name,float*amb,float*dif,
                       float*spe,float*emi,float s);
        int getmaterial(int n,float *amb,float *dif,float *spe,float*emi,float *shi);
        int setmaterialtexture(unsigned long n,unsigned int texture);
        int getmaterialtexture(unsigned long n);
        int setmaterialname(unsigned long, char*);
        int getmaterialname(unsigned long, char*);
        
        int getvertex(int n,float *x,float *y,float *z);
        int getvertexusage(int n,unsigned int *arr,int len);
        
        int translate(float x,float y,float z);
        int getboundingrect(float*x1,float*x2,float*y1,float*y2,float*z1,float*z2);
                            // x smallest, largest, y...
        int flipaxis_yz();
        
        int import_asc(char *name);
        int import_asc_adv(char *name);
 
        int getOpenGLDisplayList();
        int glDraw();
};

#endif

