/*
  Name: aMode.cpp
  Copyright: Copyright(C) 2003, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: aMode base level for game development
  
  This is not official version of the code.
*/

#include "aMode.hpp"

int aMode::init() {
    return AMODE_ERROR_INVALIDAMODE;
}
int aMode::entry() {
    return AMODE_ERROR_INVALIDAMODE;
}
int aMode::close() {
    return AMODE_ERROR_INVALIDAMODE;
}
int aMode::render() {
    return AMODE_ERROR_INVALIDAMODE;
}
int aMode::evaluate(int time) {
    return AMODE_ERROR_INVALIDAMODE;
}

aState::aState() {
    current = 0xFFFFFFFF;
    for (int i=0; i<AMODE_MAXMODES; i++) mode[i] = NULL;
    ending = false;

    time = 1;
    instance = 0;
    
    return;
}

int aState::SetTime(int t) {
    time = t;
    return 0;
}

int aState::Register(aMode *amode) {
    if (!amode) return AMODE_ERROR_INVALIDPARAM;

    bool found = false;
    int i;
    for (i=0; i<AMODE_MAXMODES; i++) {
        if (!mode[i]) {
            found = true;
            break;
        }
    }
    
    if (found) {
        mode[i] = amode;
        mode[i]->init();
    } else {
        return AMODE_ERROR_TABLEFULL;
    }
    
    return i;
};

int aState::Modify(int pos, aMode *amode) {
    if (pos < 0 || pos >= AMODE_MAXMODES) return AMODE_ERROR_INVALIDPARAM;
    
    mode[pos] = amode;
    
    return AMODE_ERROR_NOERROR;
};

int aState::set(int _m) {
    if (_m < 0 || _m >= AMODE_MAXMODES) return AMODE_ERROR_INVALIDPARAM;
    if (!mode[_m]) return AMODE_ERROR_MISSINGMODE;
    
    int lret1, lret2, ret = AMODE_SIGNAL_SUCCESS;
    
    if (current != 0xFFFFFFFF){
        lret1 = mode[current]->close();
        if (lret1 == AMODE_SIGNAL_REQUESTEND) request_end();
    }
        
    current = _m;
    lret2 = mode[current]->entry();
    if (lret2 == AMODE_SIGNAL_REQUESTEND) request_end();
    
    if (ending) ret = AMODE_SIGNAL_ENDREQUESTED;
    
    return ret;
};

int aState::get() {
    if (current == 0xFFFFFFFF) return AMODE_ERROR_NOMODESELECTED;
    return current;
};

int aState::request_end() {
    ending = true;
    return AMODE_SIGNAL_ENDREQUESTED;
}

int aState::loop() {
    int _t = GetTickCount();
    int lret1, lret2, ret = AMODE_SIGNAL_SUCCESS;

    if (current != 0xFFFFFFFF) {
        lret1 = mode[current]->evaluate(time);
        if (lret1 == AMODE_SIGNAL_REQUESTEND) request_end();
        if (lret1 == AMODE_SIGNAL_REQUESTIMMEND) request_end();
    
        if (lret1 != AMODE_SIGNAL_SUPPRESSRENDER &&
            lret1 != AMODE_SIGNAL_REQUESTIMMEND) {
            
            lret2 = mode[current]->render();
            if (lret2 == AMODE_SIGNAL_REQUESTEND) request_end();
        }
        
        if (ending) ret = AMODE_SIGNAL_ENDREQUESTED;
    }
    
    ++instance;
    
    time = GetTickCount() - _t;
    if (!time) time = 1;
    
    return ret;
};


