/*
  Name: aPath.cpp
  Copyright: Copyright(C) 2003, Jan Ringos, Tringi@Mx-3.cz
  Author: Jan Ringos, http://Tringi.Mx-3.cz
  Description: Module for examinating linearistic 3D paths 
  
  This is not official version of the code.
*/

#include "aPath.hpp"

int aPath::arealloc(int nn) {
    void *tmp = GlobalAlloc( GPTR, sizeof(double)*nn*4 );
    if (!tmp) return APATH_ERROR_MEMORY;
    
    if (data){
        CopyMemory( tmp, data, sizeof(double)*(n<?nn)*4 );
        GlobalFree(data);
    }
    data = (double*)tmp;

    n = nn;
    return 0;
}

aPath::aPath() {
    ZeroMemory(magic,8); 
    strcpy(magic,"aPath");
    compression = 0;
    n = 0;
    data = NULL;
}

aPath::~aPath() {
    if (data) GlobalFree(data);
}


int sort_function( const void *a, const void *b){
    register int i = 0;
    double *p1 = (double*)a;
    double *p2 = (double*)b;
    
    if (*p1 > *p2) i = 1;
    if (*p1 < *p2) i = -1;
    return i;
}

int aPath::addp(double ps,double v1,double v2,double v3) {
    if ( arealloc(n+1) == APATH_ERROR_MEMORY) return APATH_ERROR_MEMORY;
    
    data[4*(n-1)+0] = ps;
    data[4*(n-1)+1] = v1;
    data[4*(n-1)+2] = v2;
    data[4*(n-1)+3] = v3;
    
    if (n) {
        qsort(data,n,sizeof(double)*4,sort_function);
    }
    
    return 0;
}

float aPath::getf(double ps,float *v1,float *v2,float *v3) {
    double r;
    double v[3];
    
    r = this->get(ps, &v[0], &v[1], &v[2]);

    *v1 = (float)v[0];
    *v2 = (float)v[1];
    *v3 = (float)v[2];
    
    return (float)r;
};

double aPath::get(double ps,double *v1,double *v2,double *v3) {
    if (n <= 0){
        if (v1) *v1 = 0.0;
        if (v2) *v2 = 0.0;
        if (v3) *v3 = 0.0;
        return 0.0;
    }
    if (n == 1){
        if (v1) *v1 = data[1];
        if (v2) *v2 = data[2];
        if (v3) *v3 = data[3];
        return data[1];
    }
    if (n > 1){
        // if pos is beyond, use last value
        if (ps >= data[4*(n-1)+0]) {
            if (v1) *v1 = data[4*(n-1)+1];
            if (v2) *v2 = data[4*(n-1)+2];
            if (v3) *v3 = data[4*(n-1)+3];
            return data[4*(n-1)+1];
        }
        // if pos is before
        if (ps <= data[0]) {
            if (v1) *v1 = data[1];
            if (v2) *v2 = data[2];
            if (v3) *v3 = data[3];
            return data[1];
        }

        // look for two points where pos lies
        int a,b;
        for (a=0, b=1; b<n; ++a,++b) {
            if (ps == data[4*(b)+0]) { // 0 is determined above
                if (v1) *v1 = data[4*(b)+1];
                if (v2) *v2 = data[4*(b)+2];
                if (v3) *v3 = data[4*(b)+3];
                return data[4*(b)+1];
            }
            if (ps > data[4*(a)+0] && ps < data[4*(b)+0]) { // here we go !
                double dif = (ps-data[4*(a)+0])/(data[4*(b)+0]-data[4*(a)+0]);
                
                if (v1) *v1 = data[4*(a)+1]*(1.0-dif) + data[4*(b)+1]*dif;
                if (v2) *v2 = data[4*(a)+2]*(1.0-dif) + data[4*(b)+2]*dif;
                if (v3) *v3 = data[4*(a)+3]*(1.0-dif) + data[4*(b)+3]*dif;
                return data[4*(a)+1]*(1.0-dif) + data[4*(b)+1]*dif;
            }            
        }
    
    }
    return 0; // never here
}

double aPath::getpre(double ps,double *v1,double *v2,double *v3) {
    if (n <= 0){
        if (v1) *v1 = 0.0;
        if (v2) *v2 = 0.0;
        if (v3) *v3 = 0.0;
        return 0.0;
    }
    if (n == 1){
        if (v1) *v1 = data[1];
        if (v2) *v2 = data[2];
        if (v3) *v3 = data[3];
        return data[1];
    }
    if (n > 1){
        // if pos is beyond, use last value
        if (ps >= data[4*(n-1)+0]) {
            if (v1) *v1 = data[4*(n-1)+1];
            if (v2) *v2 = data[4*(n-1)+2];
            if (v3) *v3 = data[4*(n-1)+3];
            return data[4*(n-1)+1];
        }
        // if pos is before
        if (ps <= data[0]) {
            if (v1) *v1 = data[1];
            if (v2) *v2 = data[2];
            if (v3) *v3 = data[3];
            return data[1];
        }

        // look for two points where pos lies
        int a,b;
        for (a=0, b=1; b<n; ++a,++b) {
            if (ps == data[4*(b)+0]) { // 0 is determined above
                if (v1) *v1 = data[4*(b)+1];
                if (v2) *v2 = data[4*(b)+2];
                if (v3) *v3 = data[4*(b)+3];
                return data[4*(b)+1];
            }
            if (ps > data[4*(a)+0] && ps < data[4*(b)+0]) { // here we go !
                if (v1) *v1 = data[4*(a)+1];
                if (v2) *v2 = data[4*(a)+2];
                if (v3) *v3 = data[4*(a)+3];
                return data[4*(b)+1];
            }            
        }
    
    }
    return 0; // never here
}

double aPath::getsin(double ps,double *v1,double *v2,double *v3) {
    if (n <= 0){
        if (v1) *v1 = 0.0;
        if (v2) *v2 = 0.0;
        if (v3) *v3 = 0.0;
        return 0.0;
    }
    if (n == 1){
        if (v1) *v1 = data[1];
        if (v2) *v2 = data[2];
        if (v3) *v3 = data[3];
        return data[1];
    }
    if (n > 1){
        // if pos is beyond, use last value
        if (ps >= data[4*(n-1)+0]) {
            if (v1) *v1 = data[4*(n-1)+1];
            if (v2) *v2 = data[4*(n-1)+2];
            if (v3) *v3 = data[4*(n-1)+3];
            return data[4*(n-1)+1];
        }
        // if pos is before
        if (ps <= data[0]) {
            if (v1) *v1 = data[1];
            if (v2) *v2 = data[2];
            if (v3) *v3 = data[3];
            return data[1];
        }

        // look for two points where pos lies
        int a,b;
        for (a=0, b=1; b<n; ++a,++b) {
            if (ps == data[4*(b)+0]) { // 0 is determined above
                if (v1) *v1 = data[4*(b)+1];
                if (v2) *v2 = data[4*(b)+2];
                if (v3) *v3 = data[4*(b)+3];
                return data[4*(b)+1];
            }
            if (ps > data[4*(a)+0] && ps < data[4*(b)+0]) { // here we go !
                double f = (ps-data[4*(a)+0])/(data[4*(b)+0]-data[4*(a)+0]);
                if (f > 0.9999) {
                    if (v1) *v1 = data[4*(b)+1];
                    if (v2) *v2 = data[4*(b)+2];
                    if (v3) *v3 = data[4*(b)+3];
                    return data[4*(b)+1];
                } else
                if (f < 0.0001) {
                    if (v1) *v1 = data[4*(a)+1];
                    if (v2) *v2 = data[4*(a)+2];
                    if (v3) *v3 = data[4*(a)+3];
                    return data[4*(a)+1];
                } else {
                    f = sin( M_PI/2 + M_PI * f );
                    if (f > 0.0) f = 1.0-f;
                    if (f < 0.0) f = -1.0-f;
                    
                    if (f < 0.0) f = -f*f; else f = f*f;
    
                    if (f > 0.0) f = 1.0-f;
                    if (f < 0.0) f = -1.0-f;
                    
                    f = 0.5 - 0.5 *f;
                    
                    if (v1) *v1 = data[4*(a)+1]*(1.0-f) + data[4*(b)+1]*f;
                    if (v2) *v2 = data[4*(a)+2]*(1.0-f) + data[4*(b)+2]*f;
                    if (v3) *v3 = data[4*(a)+3]*(1.0-f) + data[4*(b)+3]*f;
                    
                    return data[4*(a)+1]*(1.0-f) + data[4*(b)+1]*f;
                }
            }            
        }
    
    }
    return 0; // never here
}


double aPath::geti(int i,double *p, double *v1, double *v2, double *v3){
    if (i < 0 || i >= getn()) {
        if (p) *p = 0.0;
        if (v1) *v1 = 0.0;
        if (v2) *v2 = 0.0;
        if (v3) *v3 = 0.0;
        return 0.0;
    }
    
    if (p) *p = data[4*(i)+0];
    if (v1) *v1 = data[4*(i)+1];
    if (v2) *v2 = data[4*(i)+2];
    if (v3) *v3 = data[4*(i)+3];
    
    return data[4*(i)+0];
}

int aPath::getn(){
    return n;
}

int aPath::seti(int i, double v1, double v2, double v3){
    if (i < 0 || i >= getn()) {
        return 0;
    }
    
    data[4*(i)+1] = v1;
    data[4*(i)+2] = v2;
    data[4*(i)+3] = v3;
    
    return 0;
}

int aPath::remi(int i){
    if (i < 0 || i >= getn()) {
        return 0;
    }
    
    if (i<n)
    for (int j=i; j<n; j++) {
        for (int d=0;d<4;d++)
            data[4*(j)+d] = data[4*(j+1)+d];
    }
    
    arealloc(n-1);
    
    return 0;
}

int aPath::save(char *name){
    FILE *f;
    f = fopen(name,"wb");
    if (!f) return APATH_ERROR_FILE;
    
    compression = 0;
    fwrite(this,16,1,f);
    fwrite(data,4*n*sizeof(double),1,f);
    
    fclose(f);
    return 0;
} // save

int aPath::save_c(char *name){
    FILE *f;
    f = fopen(name,"wb");
    if (!f) return APATH_ERROR_FILE;
    
    compression = 1;
    fwrite(this,16,1,f);

    unsigned long outLen = (int)(1.001*(float)(sizeof(double)*n*4))+32;
    char *out = (char*)GlobalAlloc(GPTR,outLen);
    if (!out) return -1;
    
    int ret = compress2( (Bytef*)out, &outLen,
                         (Bytef*)data, sizeof(double)*n*4, 9 );
    
    if (ret == Z_MEM_ERROR){ GlobalFree(out); return APATH_ERROR_MEMORY; }
    if (ret == Z_BUF_ERROR){ GlobalFree(out); return APATH_ERROR_MEMORY; }
    if (ret == Z_STREAM_ERROR){ GlobalFree(out); return APATH_ERROR_MEMORY; }
   
    fwrite(out,outLen,1,f); // hlavicku
    GlobalFree(out);
    
    fclose(f);
    return 0;
} // save

int aPath::load(char *name){
    FILE *f;
    f = fopen(name,"rb");
    if (!f) return -1;
    
    // testnout magic
    char temp[8];
    fread(temp,8,1,f);
    if (strcmp(magic,temp)){ fclose(f); return -1; }
    
    // velikost
    int _n,p1;

    fread(&_n,sizeof(int),1,f);
    if ( arealloc(_n) == APATH_ERROR_MEMORY) return APATH_ERROR_MEMORY;
    
    fread(&p1,sizeof(int),1,f);
    if (p1 > 1){ fclose(f); return APATH_ERROR_VERSION; } // nonsupported compression
    
    // decompress
    if (p1 == 0) fread(data,4*sizeof(double),n,f);
    if (p1 == 1) {
        long curpos, length;
    
        curpos = ftell(f);
        fseek(f, 0L, SEEK_END);
        int filesize = ftell(f);
        fseek(f, curpos, SEEK_SET);
    
        unsigned long insize = filesize - 16;
        char *in = (char*)GlobalAlloc(GPTR, insize );
        if (!in) return APATH_ERROR_MEMORY; // not enough memory
        
        fread(in,insize,1,f);
        
        unsigned long destLen = 4*n*sizeof(double);
        int ret = uncompress( (Bytef*)data, &destLen, (Bytef*)in, insize );
        
        if (ret == Z_MEM_ERROR){ GlobalFree(in); return APATH_ERROR_MEMORY; }
        if (ret == Z_BUF_ERROR){ GlobalFree(in); return APATH_ERROR_MEMORY; }
        if (ret == Z_DATA_ERROR){ GlobalFree(in); return APATH_ERROR_MEMORY; }
        
        if (destLen != 4*n*sizeof(double)){
            GlobalFree(in); return APATH_ERROR_FILE; 
        }
        
        GlobalFree(in);
    }
    
    fclose(f);
    
    return 0;
} // load

int aPath::reset() {
    n = 0;
    if (data) GlobalFree(data);
    data = NULL;
    return 0;
}
